import _win32typing

BG_AUTH_SCHEME_BASIC: int
BG_AUTH_SCHEME_DIGEST: int
BG_AUTH_SCHEME_NEGOTIATE: int
BG_AUTH_SCHEME_NTLM: int
BG_AUTH_SCHEME_PASSPORT: int
BG_AUTH_TARGET_PROXY: int
BG_AUTH_TARGET_SERVER: int
BG_CERT_STORE_LOCATION_CURRENT_SERVICE: int
BG_CERT_STORE_LOCATION_CURRENT_USER: int
BG_CERT_STORE_LOCATION_CURRENT_USER_GROUP_POLICY: int
BG_CERT_STORE_LOCATION_LOCAL_MACHINE: int
BG_CERT_STORE_LOCATION_LOCAL_MACHINE_ENTERPRISE: int
BG_CERT_STORE_LOCATION_LOCAL_MACHINE_GROUP_POLICY: int
BG_CERT_STORE_LOCATION_SERVICES: int
BG_CERT_STORE_LOCATION_USERS: int
BG_ERROR_CONTEXT_GENERAL_QUEUE_MANAGER: int
BG_ERROR_CONTEXT_GENERAL_TRANSPORT: int
BG_ERROR_CONTEXT_LOCAL_FILE: int
BG_ERROR_CONTEXT_NONE: int
BG_ERROR_CONTEXT_QUEUE_MANAGER_NOTIFICATION: int
BG_ERROR_CONTEXT_REMOTE_APPLICATION: int
BG_ERROR_CONTEXT_REMOTE_FILE: int
BG_ERROR_CONTEXT_UNKNOWN: int
BG_JOB_ENUM_ALL_USERS: int
BG_JOB_PRIORITY_FOREGROUND: int
BG_JOB_PRIORITY_HIGH: int
BG_JOB_PRIORITY_LOW: int
BG_JOB_PRIORITY_NORMAL: int
BG_JOB_PROXY_USAGE_AUTODETECT: int
BG_JOB_PROXY_USAGE_NO_PROXY: int
BG_JOB_PROXY_USAGE_OVERRIDE: int
BG_JOB_PROXY_USAGE_PRECONFIG: int
BG_JOB_STATE_ACKNOWLEDGED: int
BG_JOB_STATE_CANCELLED: int
BG_JOB_STATE_CONNECTING: int
BG_JOB_STATE_ERROR: int
BG_JOB_STATE_QUEUED: int
BG_JOB_STATE_SUSPENDED: int
BG_JOB_STATE_TRANSFERRED: int
BG_JOB_STATE_TRANSFERRING: int
BG_JOB_STATE_TRANSIENT_ERROR: int
BG_JOB_TYPE_DOWNLOAD: int
BG_JOB_TYPE_UPLOAD: int
BG_JOB_TYPE_UPLOAD_REPLY: int
BG_NOTIFY_DISABLE: int
BG_NOTIFY_JOB_ERROR: int
BG_NOTIFY_JOB_MODIFICATION: int
BG_NOTIFY_JOB_TRANSFERRED: int
CLSID_BackgroundCopyManager: _win32typing.PyIID
IID_IBackgroundCopyCallback: _win32typing.PyIID
IID_IBackgroundCopyError: _win32typing.PyIID
IID_IBackgroundCopyFile: _win32typing.PyIID
IID_IBackgroundCopyFile2: _win32typing.PyIID
IID_IBackgroundCopyJob: _win32typing.PyIID
IID_IBackgroundCopyJob2: _win32typing.PyIID
IID_IBackgroundCopyJob3: _win32typing.PyIID
IID_IBackgroundCopyManager: _win32typing.PyIID
IID_IEnumBackgroundCopyFiles: _win32typing.PyIID
IID_IEnumBackgroundCopyJobs: _win32typing.PyIID



from _typeshed import Incomplete
from typing import ClassVar

from ..core import WesternCalendar

class Poland(WesternCalendar):
    include_labour_day: ClassVar[bool]
    FIXED_HOLIDAYS: Incomplete
    include_easter_sunday: ClassVar[bool]
    include_easter_monday: ClassVar[bool]
    include_whit_sunday: ClassVar[bool]
    whit_sunday_label: ClassVar[str]
    include_corpus_christi: ClassVar[bool]
    include_assumption: ClassVar[bool]
    include_all_saints: ClassVar[bool]
    include_boxing_day: ClassVar[bool]



from collections.abc import Generator
from typing import Any

WHEEL_NAME: Any
NAMESPACE_PACKAGE_INIT: str

def unpack(src_dir, dst_dir) -> None: ...
def disable_info_traces() -> Generator[None, None, None]: ...

class Wheel:
    filename: Any
    def __init__(self, filename) -> None: ...
    def tags(self): ...
    def is_compatible(self): ...
    def egg_name(self): ...
    def get_dist_info(self, zf): ...
    def install_as_egg(self, destination_eggdir) -> None: ...



from networkx.utils.backends import _dispatch

@_dispatch
def maximal_matching(G): ...
@_dispatch
def is_matching(G, matching): ...
@_dispatch
def is_maximal_matching(G, matching): ...
@_dispatch
def is_perfect_matching(G, matching): ...
@_dispatch
def min_weight_matching(G, weight: str = "weight"): ...
@_dispatch
def max_weight_matching(G, maxcardinality: bool = False, weight: str = "weight"): ...



import mailbox

DEFAULT_MAXMEM: int

class mbox_readonlydir(mailbox.mbox):
    maxmem: int
    def __init__(self, path: str, factory: type | None = None, create: bool = True, maxmem: int = 1048576) -> None: ...
    def flush(self) -> None: ...



from _typeshed import Incomplete
from typing import ClassVar

from ..core import OrthodoxCalendar

class Russia(OrthodoxCalendar):
    include_labour_day: ClassVar[bool]
    FIXED_HOLIDAYS: Incomplete
    include_christmas: ClassVar[bool]
    covid19_2020_start: Incomplete
    covid19_2020_end: Incomplete
    labour_day_label: ClassVar[str]
    def get_fixed_holidays(self, year): ...
    def get_calendar_holidays(self, year): ...
    def is_working_day(self, day, extra_working_days: Incomplete | None = None, extra_holidays: Incomplete | None = None): ...



from _typeshed import Unused
from datetime import date, datetime, time, timedelta

from MySQLdb._mysql import string_literal as string_literal

Date = date
Time = time
TimeDelta = timedelta
Timestamp = datetime
DateTimeDeltaType = timedelta
DateTimeType = datetime

def DateFromTicks(ticks: float | None) -> date: ...
def TimeFromTicks(ticks: float | None) -> time: ...
def TimestampFromTicks(ticks: float | None) -> datetime: ...

format_TIME = str
format_DATE = str

def format_TIMEDELTA(v: timedelta) -> str: ...
def format_TIMESTAMP(d: datetime) -> str: ...
def DateTime_or_None(s: str) -> datetime | None: ...
def TimeDelta_or_None(s: str) -> timedelta | None: ...
def Time_or_None(s: str) -> time | None: ...
def Date_or_None(s: str) -> date | None: ...
def DateTime2literal(d: datetime, c: Unused) -> str: ...
def DateTimeDelta2literal(d: datetime, c: Unused) -> str: ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

__all__ = ["treewidth_min_degree", "treewidth_min_fill_in"]

@_dispatch
def treewidth_min_degree(G): ...
@_dispatch
def treewidth_min_fill_in(G): ...

class MinDegreeHeuristic:
    count: Incomplete
    def __init__(self, graph) -> None: ...
    def best_node(self, graph): ...



from _typeshed import Incomplete

from .icalendar import VCalendar2_0

class HCalendar(VCalendar2_0):
    name: str
    @classmethod
    def serialize(cls, obj, buf: Incomplete | None = None, lineLength: Incomplete | None = None, validate: bool = True): ...



from typing import Any

from passlib.context import CryptContext

linux_context: Any
linux2_context: Any
freebsd_context: Any
openbsd_context: Any
netbsd_context: Any
# Only exists if crypt is present
host_context: CryptContext



from _typeshed import Incomplete

from boltons.dictutils import OrderedMultiDict

SCHEME_PORT_MAP: Incomplete
NO_NETLOC_SCHEMES: Incomplete

class URLParseError(ValueError): ...

DEFAULT_ENCODING: str

def to_unicode(obj: object) -> str: ...
def find_all_links(text, with_text: bool = False, default_scheme: str = "https", schemes=()): ...
def quote_path_part(text, full_quote: bool = True): ...
def quote_query_part(text, full_quote: bool = True): ...
def quote_fragment_part(text, full_quote: bool = True): ...
def quote_userinfo_part(text, full_quote: bool = True): ...
def unquote(string, encoding: str = "utf-8", errors: str = "replace"): ...
def unquote_to_bytes(string): ...
def register_scheme(text, uses_netloc: Incomplete | None = None, default_port: Incomplete | None = None) -> None: ...
def resolve_path_parts(path_parts): ...

class cachedproperty:
    __doc__: Incomplete
    func: Incomplete
    def __init__(self, func) -> None: ...
    def __get__(self, obj, objtype: Incomplete | None = None): ...

class URL:
    scheme: Incomplete
    username: Incomplete
    password: Incomplete
    family: Incomplete
    host: Incomplete
    port: Incomplete
    path_parts: Incomplete
    fragment: Incomplete
    def __init__(self, url: str = "") -> None: ...
    @classmethod
    def from_parts(
        cls,
        scheme: Incomplete | None = None,
        host: Incomplete | None = None,
        path_parts=(),
        query_params=(),
        fragment: str = "",
        port: Incomplete | None = None,
        username: Incomplete | None = None,
        password: Incomplete | None = None,
    ): ...
    query_params: Incomplete
    qp: Incomplete
    @property
    def path(self): ...
    @path.setter
    def path(self, path_text) -> None: ...
    @property
    def uses_netloc(self): ...
    @property
    def default_port(self): ...
    def normalize(self, with_case: bool = True) -> None: ...
    def navigate(self, dest): ...
    def get_authority(self, full_quote: bool = False, with_userinfo: bool = False): ...
    def to_text(self, full_quote: bool = False): ...
    def __unicode__(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...

def parse_host(host): ...
def parse_url(url_text): ...

DEFAULT_PARSED_URL: Incomplete

def parse_qsl(qs, keep_blank_values: bool = True, encoding="utf8"): ...

PREV: Incomplete
NEXT: Incomplete
KEY: Incomplete
VALUE: Incomplete
SPREV: Incomplete
SNEXT: Incomplete

class QueryParamDict(OrderedMultiDict[Incomplete, Incomplete]):
    @classmethod
    def from_text(cls, query_string): ...
    def to_text(self, full_quote: bool = False): ...



from _typeshed import Incomplete, Unused
from typing import ClassVar, Literal

from openpyxl.chart.axis import ChartLines
from openpyxl.descriptors.base import Typed
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable

class UpDownBars(Serialisable):
    tagname: ClassVar[str]
    gapWidth: Incomplete
    upBars: Typed[ChartLines, Literal[True]]
    downBars: Typed[ChartLines, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self, gapWidth: int = 150, upBars: ChartLines | None = None, downBars: ChartLines | None = None, extLst: Unused = None
    ) -> None: ...



import sys
from _typeshed import ReadableBuffer, Unused
from typing import IO, Any, BinaryIO, Literal, NamedTuple, NoReturn, overload
from typing_extensions import Self, TypeAlias, deprecated

if sys.version_info >= (3, 9):
    __all__ = ["open", "Error", "Wave_read", "Wave_write"]
else:
    __all__ = ["open", "openfp", "Error", "Wave_read", "Wave_write"]

_File: TypeAlias = str | IO[bytes]

class Error(Exception): ...

WAVE_FORMAT_PCM: Literal[1]

class _wave_params(NamedTuple):
    nchannels: int
    sampwidth: int
    framerate: int
    nframes: int
    comptype: str
    compname: str

class Wave_read:
    def __init__(self, f: _File) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *args: Unused) -> None: ...
    def __del__(self) -> None: ...
    def getfp(self) -> BinaryIO | None: ...
    def rewind(self) -> None: ...
    def close(self) -> None: ...
    def tell(self) -> int: ...
    def getnchannels(self) -> int: ...
    def getnframes(self) -> int: ...
    def getsampwidth(self) -> int: ...
    def getframerate(self) -> int: ...
    def getcomptype(self) -> str: ...
    def getcompname(self) -> str: ...
    def getparams(self) -> _wave_params: ...
    @deprecated("Deprecated in Python 3.13; removal scheduled for Python 3.15")
    def getmarkers(self) -> None: ...
    @deprecated("Deprecated in Python 3.13; removal scheduled for Python 3.15")
    def getmark(self, id: Any) -> NoReturn: ...
    def setpos(self, pos: int) -> None: ...
    def readframes(self, nframes: int) -> bytes: ...

class Wave_write:
    def __init__(self, f: _File) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *args: Unused) -> None: ...
    def __del__(self) -> None: ...
    def setnchannels(self, nchannels: int) -> None: ...
    def getnchannels(self) -> int: ...
    def setsampwidth(self, sampwidth: int) -> None: ...
    def getsampwidth(self) -> int: ...
    def setframerate(self, framerate: float) -> None: ...
    def getframerate(self) -> int: ...
    def setnframes(self, nframes: int) -> None: ...
    def getnframes(self) -> int: ...
    def setcomptype(self, comptype: str, compname: str) -> None: ...
    def getcomptype(self) -> str: ...
    def getcompname(self) -> str: ...
    def setparams(self, params: _wave_params | tuple[int, int, int, int, str, str]) -> None: ...
    def getparams(self) -> _wave_params: ...
    @deprecated("Deprecated in Python 3.13; removal scheduled for Python 3.15")
    def setmark(self, id: Any, pos: Any, name: Any) -> NoReturn: ...
    @deprecated("Deprecated in Python 3.13; removal scheduled for Python 3.15")
    def getmark(self, id: Any) -> NoReturn: ...
    @deprecated("Deprecated in Python 3.13; removal scheduled for Python 3.15")
    def getmarkers(self) -> None: ...
    def tell(self) -> int: ...
    def writeframesraw(self, data: ReadableBuffer) -> None: ...
    def writeframes(self, data: ReadableBuffer) -> None: ...
    def close(self) -> None: ...

@overload
def open(f: _File, mode: Literal["r", "rb"]) -> Wave_read: ...
@overload
def open(f: _File, mode: Literal["w", "wb"]) -> Wave_write: ...
@overload
def open(f: _File, mode: str | None = None) -> Any: ...

if sys.version_info < (3, 9):
    openfp = open



from typing import Any

CS_ESCAPE: Any
FIRST_ELEMENT: Any

def make_charset(letters): ...
def regex_opt_inner(strings, open_paren): ...
def regex_opt(strings, prefix: str = "", suffix: str = ""): ...



from _typeshed import Incomplete

class ValidationErrorCollection:
    data: dict[str, Incomplete]
    def __init__(self, data: Incomplete | None = None) -> None: ...
    @property
    def deep_errors(self): ...
    def for_index(self, index): ...
    def for_object(self, nested_key): ...
    def on(self, attribute): ...
    @property
    def deep_size(self): ...
    @property
    def errors(self): ...
    @property
    def size(self): ...
    def __getitem__(self, index): ...
    def __len__(self) -> int: ...



import threading
from typing import Any

TRUTHY_STRINGS: Any

def asbool(value): ...

DEBUG: Any

def code_info(*args, **kwargs) -> None: ...
def decorator(*args, **kwargs) -> None: ...
def frame_info(*args, **kwargs) -> None: ...
def func_info(*args, **kwargs) -> None: ...
def gen_info(*args, **kwargs) -> None: ...
def get_stack(*args, **kwargs) -> None: ...
def logging_debug(log, message, *args, **kwargs) -> None: ...

class keyword_only:
    defaults: Any
    def __init__(self, **kwargs) -> None: ...
    def __call__(self, wrapped): ...

def positional(max_pos_args): ...

threading_local = threading.local

def tweak_logging(*args, **kwargs) -> None: ...
def wrapping(*args, **kwargs) -> None: ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def hopcroft_karp_matching(G, top_nodes: Incomplete | None = None): ...
@_dispatch
def eppstein_matching(G, top_nodes: Incomplete | None = None): ...
@_dispatch
def to_vertex_cover(G, matching, top_nodes: Incomplete | None = None): ...

maximum_matching = hopcroft_karp_matching

@_dispatch
def minimum_weight_full_matching(G, top_nodes: Incomplete | None = None, weight: str = "weight"): ...



from typing import Any

from .. import Command

class rotate(Command):
    description: str
    user_options: Any
    boolean_options: list[str]
    match: Any
    dist_dir: Any
    keep: Any
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    def run(self) -> None: ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def margulis_gabber_galil_graph(n, create_using: Incomplete | None = None): ...
@_dispatch
def chordal_cycle_graph(p, create_using: Incomplete | None = None): ...
@_dispatch
def paley_graph(p, create_using: Incomplete | None = None): ...



from _typeshed import Incomplete
from typing import Any

from oauthlib.oauth2.rfc6749.endpoints.base import BaseEndpoint as BaseEndpoint

log: Any

class UserInfoEndpoint(BaseEndpoint):
    bearer: Any
    request_validator: Any
    def __init__(self, request_validator) -> None: ...
    def create_userinfo_response(
        self, uri, http_method: str = "GET", body: Incomplete | None = None, headers: Incomplete | None = None
    ): ...
    def validate_userinfo_request(self, request) -> None: ...



from _typeshed import Incomplete
from collections.abc import Sequence
from dataclasses import dataclass
from typing import Final, Literal, TypedDict, type_check_only

MAX_DEPTH: Final = 125

@type_check_only
class _BracketInfo(TypedDict):
    pair: str
    type: Literal["o", "c"]

BIDI_BRACKETS: Final[dict[str, _BracketInfo]]

class BidiCharacter:
    character_index: int
    character: str
    bidi_class: str
    original_bidi_class: str
    embedding_level: str
    direction: Incomplete | None

    def __init__(self, character_index: int, character: str, embedding_level: str, debug: bool) -> None: ...
    def get_direction_from_level(self) -> Literal["L", "R"]: ...
    def set_class(self, cls: str) -> None: ...

@dataclass
class DirectionalStatus:
    embedding_level: int  # between 0 and MAX_DEPTH
    directional_override_status: Literal["N", "L", "R"]
    directional_isolate_status: bool

class IsolatingRun:
    characters: list[BidiCharacter]
    previous_direction: str
    next_direction: str
    def __init__(self, characters: list[BidiCharacter], sos: str, eos: str) -> None: ...
    def resolve_weak_types(self) -> None: ...
    def pair_brackets(self) -> list[tuple[int, int]]: ...
    def resolve_neutral_types(self) -> None: ...
    def resolve_implicit_levels(self) -> None: ...

def auto_detect_base_direction(string: str, stop_at_pdi: bool = False, debug: bool = False) -> Literal["L", "R"]: ...
def calculate_isolate_runs(paragraph: Sequence[BidiCharacter]) -> list[IsolatingRun]: ...

class BidiParagraph:
    text: str
    base_direction: Literal["L", "R"]
    debug: bool
    base_embedding_level: int
    characters: list[BidiCharacter]

    def __init__(self, text: str, base_direction: Literal["L", "R"] | None = None, debug: bool = False) -> None: ...
    def get_characters(self) -> list[BidiCharacter]: ...
    def get_characters_with_embedding_level(self) -> list[BidiCharacter]: ...
    def get_reordered_characters(self) -> list[BidiCharacter]: ...
    def get_all(self) -> tuple[list[BidiCharacter], tuple[BidiCharacter, ...]]: ...
    def get_reordered_string(self) -> str: ...
    def get_bidi_fragments(self) -> tuple[tuple[str, Literal["L", "R"]], ...]: ...
    def get_bidi_characters(self) -> None: ...
    def split_bidi_fragments(self) -> tuple[tuple[str, Literal["L", "R"]], ...]: ...
    def reorder_resolved_levels(self) -> tuple[BidiCharacter, ...]: ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def floyd_warshall_numpy(G, nodelist: Incomplete | None = None, weight: str = "weight"): ...
@_dispatch
def floyd_warshall_predecessor_and_distance(G, weight: str = "weight"): ...
@_dispatch
def reconstruct_path(source, target, predecessors): ...
@_dispatch
def floyd_warshall(G, weight: str = "weight"): ...



from _typeshed import Incomplete
from typing import Any, overload
from typing_extensions import TypeAlias

from stripe.stripe_object import StripeObject
from stripe.stripe_response import StripeResponse

def utf8(value): ...
def log_debug(message, **params) -> None: ...
def log_info(message, **params) -> None: ...
def dashboard_link(request_id): ...
def logfmt(props): ...

class class_method_variant:
    class_method_name: Any
    def __init__(self, class_method_name) -> None: ...
    method: Any
    def __call__(self, method): ...
    def __get__(self, obj, objtype: Incomplete | None = None): ...

@overload
def populate_headers(idempotency_key: None) -> None: ...
@overload
def populate_headers(idempotency_key: str) -> dict[str, str]: ...

_RespType: TypeAlias = dict[Any, Any] | StripeObject | StripeResponse

# undocumented
@overload
def convert_to_stripe_object(
    resp: list[Any],
    api_key: Incomplete | None = None,
    stripe_version: Incomplete | None = None,
    stripe_account: Incomplete | None = None,
) -> list[Any]: ...
@overload
def convert_to_stripe_object(
    resp: _RespType,
    api_key: Incomplete | None = None,
    stripe_version: Incomplete | None = None,
    stripe_account: Incomplete | None = None,
) -> StripeObject: ...



from pyasn1.codec.ber.encoder import AbstractItemEncoder
from pyasn1.codec.cer import encoder
from pyasn1.type.tag import TagSet

__all__ = ["Encoder", "encode"]

class SetEncoder(encoder.SetEncoder): ...

TAG_MAP: dict[TagSet, AbstractItemEncoder]
TYPE_MAP: dict[int, AbstractItemEncoder]
# deprecated aliases
tagMap = TAG_MAP
typeMap = TYPE_MAP

class SingleItemEncoder(encoder.SingleItemEncoder):
    fixedDefLengthMode: bool
    fixedChunkSize: int

    TAG_MAP: dict[TagSet, AbstractItemEncoder]
    TYPE_MAP: dict[int, AbstractItemEncoder]

class Encoder(encoder.Encoder):
    SINGLE_ITEM_ENCODER: type[SingleItemEncoder]

encode: Encoder



from networkx.utils.backends import _dispatch

def tree_data(G, root, ident: str = "id", children: str = "children"): ...
@_dispatch
def tree_graph(data, ident: str = "id", children: str = "children"): ...



from collections.abc import Iterable

DEFAULT_MOUNT_POINT: str
ALLOWED_ROLE_TYPES: Iterable[str]
ALLOWED_SECRETS_TYPES: Iterable[str]
SERVICE_ACCOUNT_KEY_ALGORITHMS: Iterable[str]
SERVICE_ACCOUNT_KEY_TYPES: Iterable[str]
GCP_CERTS_ENDPOINT: str



from typing import Any

OID_CONTROL: str
OID_EXTENSION: str
OID_FEATURE: str
OID_UNSOLICITED_NOTICE: str
OID_ATTRIBUTE_TYPE: str
OID_DIT_CONTENT_RULE: str
OID_LDAP_URL_EXTENSION: str
OID_FAMILY: str
OID_MATCHING_RULE: str
OID_NAME_FORM: str
OID_OBJECT_CLASS: str
OID_ADMINISTRATIVE_ROLE: str
OID_LDAP_SYNTAX: str
CLASS_STRUCTURAL: str
CLASS_ABSTRACT: str
CLASS_AUXILIARY: str
ATTRIBUTE_USER_APPLICATION: str
ATTRIBUTE_DIRECTORY_OPERATION: str
ATTRIBUTE_DISTRIBUTED_OPERATION: str
ATTRIBUTE_DSA_OPERATION: str

def constant_to_oid_kind(oid_kind): ...
def decode_oids(sequence): ...
def decode_syntax(syntax): ...
def oid_to_string(oid): ...

Oids: Any



from abc import abstractmethod
from collections.abc import Callable, Iterable
from threading import local
from types import ModuleType
from typing import Literal, NewType
from typing_extensions import TypeAlias

from fanstatic.compiler import Compiler, Minifier

_Renderer: TypeAlias = Callable[[str], str]

DEFAULT_SIGNATURE: str
VERSION_PREFIX: str
BUNDLE_PREFIX: str
NEEDED: str
DEBUG: str
MINIFIED: str

def set_resource_file_existence_checking(v: bool) -> None: ...
def set_auto_register_library(v: bool) -> None: ...

class UnknownResourceExtensionError(Exception): ...
class ModeResourceDependencyError(Exception): ...

UnknownResourceExtension = UnknownResourceExtensionError

class UnknownResourceError(Exception): ...
class ConfigurationError(Exception): ...
class LibraryDependencyCycleError(Exception): ...
class SlotError(Exception): ...

class Library:
    path: str
    name: str
    rootpath: str
    ignores: list[str]
    version: str | None
    known_resources: dict[str, Resource]
    known_assets: dict[str, Asset]
    module: ModuleType
    compilers: dict[str, Compiler]
    minifiers: dict[str, Minifier]
    def __init__(
        self,
        name: str,
        rootpath: str,
        ignores: list[str] | None = None,
        version: str | None = None,
        compilers: dict[str, Compiler] | None = None,
        minifiers: dict[str, Minifier] | None = None,
    ) -> None: ...
    def check_dependency_cycle(self, resource: Resource) -> None: ...
    def register(self, resource: Resource) -> None: ...
    def signature(self, recompute_hashes: bool = False, version_method: Callable[[str], str] | None = None) -> str: ...

def caller_dir() -> str: ...

class InclusionRenderers(dict[str, tuple[int, _Renderer]]):
    def register(self, extension: str, renderer: _Renderer, order: int | None = None) -> None: ...

inclusion_renderers: InclusionRenderers

def register_inclusion_renderer(extension: str, renderer: _Renderer, order: int | None = None) -> None: ...
def render_ico(url: str) -> str: ...
def render_css(url: str) -> str: ...
def render_js(url: str) -> str: ...
def render_print_css(url: str) -> str: ...
def render_screen_css(url: str) -> str: ...

class Renderable:
    @abstractmethod
    def render(self, library_url: str) -> str: ...

class Dependable:
    @property
    @abstractmethod
    def resources(self) -> set[Dependable]: ...
    @property
    @abstractmethod
    def depends(self) -> set[Dependable]: ...
    @property
    @abstractmethod
    def supports(self) -> set[Dependable]: ...
    def add_dependency(self, dependency: Dependable) -> None: ...
    @abstractmethod
    def set_dependencies(self, dependencies: Iterable[Dependable] | None) -> None: ...
    @abstractmethod
    def list_assets(self) -> set[Asset]: ...
    def list_supporting(self) -> set[Dependable]: ...

class Asset(Dependable):
    resources: set[Dependable]
    depends: set[Dependable]
    supports: set[Dependable]
    library: Library
    def __init__(self, library: Library, depends: Iterable[Dependable] | None = None) -> None: ...
    def set_dependencies(self, depends: Iterable[Dependable] | None) -> None: ...
    def list_assets(self) -> set[Asset]: ...

_NothingType = NewType("_NothingType", object)
NOTHING: _NothingType

class Resource(Renderable, Asset):
    relpath: str
    ext: str
    mode_parent: str | None
    compiler: Compiler
    source: str | None
    minifier: Minifier
    minified: Resource | None
    bottom: bool
    dont_bundle: bool
    renderer: _Renderer
    modes: dict[str, Resource]
    supersedes: list[Resource]
    rollups: list[Resource]
    def __init__(
        self,
        library: Library,
        relpath: str,
        depends: Iterable[Dependable] | None = None,
        supersedes: list[Resource] | None = None,
        bottom: bool = False,
        renderer: _Renderer | None = None,
        debug: str | Resource | None = None,
        dont_bundle: bool = False,
        minified: str | Resource | None = None,
        minifier: Minifier | _NothingType = ...,
        compiler: Compiler | _NothingType = ...,
        source: str | None = None,
        mode_parent: str | None = None,
    ) -> None: ...
    def fullpath(self, path: str | None = None) -> str: ...
    def compile(self, force: bool = False) -> None: ...
    def render(self, library_url: str) -> str: ...
    def mode(self, mode: str | None) -> Resource: ...
    def need(self, slots: dict[Slot, Resource] | None = None) -> None: ...

_RequiredDefaultMarkerType = NewType("_RequiredDefaultMarkerType", object)
REQUIRED_DEFAULT_MARKER: _RequiredDefaultMarkerType

class Slot(Asset):
    default: Resource | None
    ext: str
    required: bool
    def __init__(
        self,
        library: Library,
        extension: str,
        depends: Iterable[Dependable] | None = None,
        required: bool | _RequiredDefaultMarkerType = ...,
        default: Resource | None = None,
    ) -> None: ...

class FilledSlot(Renderable):
    filledby: Resource
    library: Library
    relpath: str
    bottom: bool
    rollups: list[Resource]
    dont_bundle: bool
    ext: str
    order: int
    renderer: _Renderer
    dependency_nr: int
    modes: dict[str, FilledSlot]
    def __init__(self, slot: Slot, resource: Resource) -> None: ...
    def render(self, library_url: str) -> str: ...
    def compile(self, force: bool = False) -> None: ...
    def mode(self, mode: str | None) -> FilledSlot: ...

class Group(Dependable):
    resources: set[Dependable]
    depends: set[Dependable]
    supports: set[Dependable]
    def __init__(self, depends: Iterable[Dependable]) -> None: ...
    def set_dependencies(self, depends: Iterable[Dependable]) -> None: ...  # type: ignore[override]
    def list_assets(self) -> set[Asset]: ...
    def need(self, slots: dict[Slot, Resource] | None = None) -> None: ...

GroupResource = Group

class NeededResources:
    def __init__(
        self,
        versioning: bool = False,
        versioning_use_md5: bool = False,
        recompute_hashes: bool = True,
        base_url: str | None = None,
        script_name: str | None = None,
        publisher_signature: str = ...,
        resources: Iterable[Dependable] | None = None,
    ) -> None: ...
    def has_resources(self) -> bool: ...
    def has_base_url(self) -> bool: ...
    def set_base_url(self, url: str) -> None: ...
    def need(self, resource: Resource | Group, slots: dict[Slot, Resource] | None = None) -> None: ...
    def resources(self) -> set[Resource]: ...
    def clear(self) -> None: ...
    def library_url(self, library: Library) -> str: ...

class DummyNeededResources:
    def need(self, resource: Resource | Group, slots: dict[Slot, Resource] | None = None) -> None: ...
    def has_resources(self) -> Literal[False]: ...

thread_local_needed_data: local

def init_needed(
    versioning: bool = False,
    versioning_use_md5: bool = False,
    recompute_hashes: bool = True,
    base_url: str | None = None,
    script_name: str | None = None,
    publisher_signature: str = ...,
    resources: Iterable[Dependable] | None = None,
) -> NeededResources: ...
def del_needed() -> None: ...
def get_needed() -> NeededResources | DummyNeededResources: ...
def clear_needed() -> None: ...

class Bundle(Renderable):
    def __init__(self) -> None: ...
    @property
    def dirname(self) -> str: ...
    @property
    def library(self) -> Library: ...
    @property
    def renderer(self) -> _Renderer: ...
    @property
    def ext(self) -> str: ...
    @property
    def relpath(self) -> str: ...
    def resources(self) -> list[Resource]: ...
    def render(self, library_url: str) -> str: ...
    def fits(self, resource: Resource) -> bool: ...
    def append(self, resource: Resource) -> None: ...
    def add_to_list(self, result: list[Renderable]) -> None: ...



from _typeshed import Incomplete
from decimal import Decimal
from typing import Final

from braintree.add_on import AddOn
from braintree.descriptor import Descriptor
from braintree.discount import Discount
from braintree.resource import Resource
from braintree.subscription_status_event import SubscriptionStatusEvent
from braintree.transaction import Transaction

class Subscription(Resource):
    class TrialDurationUnit:
        Day: Final = "day"
        Month: Final = "month"

    class Source:
        Api: Final = "api"
        ControlPanel: Final = "control_panel"
        Recurring: Final = "recurring"

    class Status:
        Active: Final = "Active"
        Canceled: Final = "Canceled"
        Expired: Final = "Expired"
        PastDue: Final = "Past Due"
        Pending: Final = "Pending"

    @staticmethod
    def create(params: Incomplete | None = None): ...
    @staticmethod
    def create_signature(): ...
    @staticmethod
    def find(subscription_id): ...
    @staticmethod
    def retry_charge(subscription_id, amount: Incomplete | None = None, submit_for_settlement: bool = False): ...
    @staticmethod
    def update(subscription_id, params: Incomplete | None = None): ...
    @staticmethod
    def cancel(subscription_id): ...
    @staticmethod
    def search(*query): ...
    @staticmethod
    def update_signature(): ...
    price: Decimal
    balance: Decimal
    next_billing_period_amount: Decimal
    add_ons: list[AddOn]
    descriptor: Descriptor
    description: Incomplete
    discounts: list[Discount]
    status_history: list[SubscriptionStatusEvent]
    transactions: list[Transaction]
    def __init__(self, gateway, attributes) -> None: ...



import argparse
import ast
from _typeshed import Incomplete
from collections.abc import Generator
from typing import Any, ClassVar

class Plugin:
    name: ClassVar[str]
    version: ClassVar[str]
    @staticmethod
    def add_options(option_manager: Any) -> None: ...
    @classmethod
    def parse_options(cls, options: argparse.Namespace) -> None: ...
    def __init__(self, tree: ast.AST) -> None: ...
    def run(self) -> Generator[tuple[int, int, str, type[Any]], None, None]: ...

def __getattr__(name: str) -> Incomplete: ...  # incomplete (other attributes are normally not accessed)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import s2clientprotocol.common_pb2

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _DisplayType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _DisplayTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_DisplayType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    Visible: _DisplayType.ValueType  # 1
    """Fully visible"""
    Snapshot: _DisplayType.ValueType  # 2
    """Dimmed version of unit left behind after entering fog of war"""
    Hidden: _DisplayType.ValueType  # 3
    """Fully hidden"""
    Placeholder: _DisplayType.ValueType  # 4
    """Building that hasn't started construction."""

class DisplayType(_DisplayType, metaclass=_DisplayTypeEnumTypeWrapper): ...

Visible: DisplayType.ValueType  # 1
"""Fully visible"""
Snapshot: DisplayType.ValueType  # 2
"""Dimmed version of unit left behind after entering fog of war"""
Hidden: DisplayType.ValueType  # 3
"""Fully hidden"""
Placeholder: DisplayType.ValueType  # 4
"""Building that hasn't started construction."""
global___DisplayType = DisplayType

class _Alliance:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _AllianceEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Alliance.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    Self: _Alliance.ValueType  # 1
    Ally: _Alliance.ValueType  # 2
    Neutral: _Alliance.ValueType  # 3
    Enemy: _Alliance.ValueType  # 4

class Alliance(_Alliance, metaclass=_AllianceEnumTypeWrapper): ...

Self: Alliance.ValueType  # 1
Ally: Alliance.ValueType  # 2
Neutral: Alliance.ValueType  # 3
Enemy: Alliance.ValueType  # 4
global___Alliance = Alliance

class _CloakState:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _CloakStateEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_CloakState.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    CloakedUnknown: _CloakState.ValueType  # 0
    """Under the fog, so unknown whether it's cloaked or not."""
    Cloaked: _CloakState.ValueType  # 1
    CloakedDetected: _CloakState.ValueType  # 2
    NotCloaked: _CloakState.ValueType  # 3
    CloakedAllied: _CloakState.ValueType  # 4

class CloakState(_CloakState, metaclass=_CloakStateEnumTypeWrapper): ...

CloakedUnknown: CloakState.ValueType  # 0
"""Under the fog, so unknown whether it's cloaked or not."""
Cloaked: CloakState.ValueType  # 1
CloakedDetected: CloakState.ValueType  # 2
NotCloaked: CloakState.ValueType  # 3
CloakedAllied: CloakState.ValueType  # 4
global___CloakState = CloakState

@typing_extensions.final
class StartRaw(google.protobuf.message.Message):
    """
    Start
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MAP_SIZE_FIELD_NUMBER: builtins.int
    PATHING_GRID_FIELD_NUMBER: builtins.int
    TERRAIN_HEIGHT_FIELD_NUMBER: builtins.int
    PLACEMENT_GRID_FIELD_NUMBER: builtins.int
    PLAYABLE_AREA_FIELD_NUMBER: builtins.int
    START_LOCATIONS_FIELD_NUMBER: builtins.int
    @property
    def map_size(self) -> s2clientprotocol.common_pb2.Size2DI:
        """Width and height of the map."""
    @property
    def pathing_grid(self) -> s2clientprotocol.common_pb2.ImageData:
        """1 bit bitmap of the pathing grid."""
    @property
    def terrain_height(self) -> s2clientprotocol.common_pb2.ImageData:
        """1 byte bitmap of the terrain height."""
    @property
    def placement_grid(self) -> s2clientprotocol.common_pb2.ImageData:
        """1 bit bitmap of the building placement grid."""
    @property
    def playable_area(self) -> s2clientprotocol.common_pb2.RectangleI:
        """The playable cells."""
    @property
    def start_locations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[s2clientprotocol.common_pb2.Point2D]:
        """Possible start locations for players."""
    def __init__(
        self,
        *,
        map_size: s2clientprotocol.common_pb2.Size2DI | None = ...,
        pathing_grid: s2clientprotocol.common_pb2.ImageData | None = ...,
        terrain_height: s2clientprotocol.common_pb2.ImageData | None = ...,
        placement_grid: s2clientprotocol.common_pb2.ImageData | None = ...,
        playable_area: s2clientprotocol.common_pb2.RectangleI | None = ...,
        start_locations: collections.abc.Iterable[s2clientprotocol.common_pb2.Point2D] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["map_size", b"map_size", "pathing_grid", b"pathing_grid", "placement_grid", b"placement_grid", "playable_area", b"playable_area", "terrain_height", b"terrain_height"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["map_size", b"map_size", "pathing_grid", b"pathing_grid", "placement_grid", b"placement_grid", "playable_area", b"playable_area", "start_locations", b"start_locations", "terrain_height", b"terrain_height"]) -> None: ...

global___StartRaw = StartRaw

@typing_extensions.final
class ObservationRaw(google.protobuf.message.Message):
    """
    Observation
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PLAYER_FIELD_NUMBER: builtins.int
    UNITS_FIELD_NUMBER: builtins.int
    MAP_STATE_FIELD_NUMBER: builtins.int
    EVENT_FIELD_NUMBER: builtins.int
    EFFECTS_FIELD_NUMBER: builtins.int
    RADAR_FIELD_NUMBER: builtins.int
    @property
    def player(self) -> global___PlayerRaw: ...
    @property
    def units(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Unit]: ...
    @property
    def map_state(self) -> global___MapState:
        """Fog of war, creep and so on. Board stuff that changes per frame."""
    @property
    def event(self) -> global___Event: ...
    @property
    def effects(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Effect]: ...
    @property
    def radar(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___RadarRing]: ...
    def __init__(
        self,
        *,
        player: global___PlayerRaw | None = ...,
        units: collections.abc.Iterable[global___Unit] | None = ...,
        map_state: global___MapState | None = ...,
        event: global___Event | None = ...,
        effects: collections.abc.Iterable[global___Effect] | None = ...,
        radar: collections.abc.Iterable[global___RadarRing] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["event", b"event", "map_state", b"map_state", "player", b"player"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["effects", b"effects", "event", b"event", "map_state", b"map_state", "player", b"player", "radar", b"radar", "units", b"units"]) -> None: ...

global___ObservationRaw = ObservationRaw

@typing_extensions.final
class RadarRing(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    POS_FIELD_NUMBER: builtins.int
    RADIUS_FIELD_NUMBER: builtins.int
    @property
    def pos(self) -> s2clientprotocol.common_pb2.Point: ...
    radius: builtins.float
    def __init__(
        self,
        *,
        pos: s2clientprotocol.common_pb2.Point | None = ...,
        radius: builtins.float | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pos", b"pos", "radius", b"radius"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pos", b"pos", "radius", b"radius"]) -> None: ...

global___RadarRing = RadarRing

@typing_extensions.final
class PowerSource(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    POS_FIELD_NUMBER: builtins.int
    RADIUS_FIELD_NUMBER: builtins.int
    TAG_FIELD_NUMBER: builtins.int
    @property
    def pos(self) -> s2clientprotocol.common_pb2.Point: ...
    radius: builtins.float
    tag: builtins.int
    def __init__(
        self,
        *,
        pos: s2clientprotocol.common_pb2.Point | None = ...,
        radius: builtins.float | None = ...,
        tag: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pos", b"pos", "radius", b"radius", "tag", b"tag"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pos", b"pos", "radius", b"radius", "tag", b"tag"]) -> None: ...

global___PowerSource = PowerSource

@typing_extensions.final
class PlayerRaw(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    POWER_SOURCES_FIELD_NUMBER: builtins.int
    CAMERA_FIELD_NUMBER: builtins.int
    UPGRADE_IDS_FIELD_NUMBER: builtins.int
    @property
    def power_sources(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___PowerSource]: ...
    @property
    def camera(self) -> s2clientprotocol.common_pb2.Point: ...
    @property
    def upgrade_ids(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """TODO: Add to UI observation?"""
    def __init__(
        self,
        *,
        power_sources: collections.abc.Iterable[global___PowerSource] | None = ...,
        camera: s2clientprotocol.common_pb2.Point | None = ...,
        upgrade_ids: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["camera", b"camera"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["camera", b"camera", "power_sources", b"power_sources", "upgrade_ids", b"upgrade_ids"]) -> None: ...

global___PlayerRaw = PlayerRaw

@typing_extensions.final
class UnitOrder(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ABILITY_ID_FIELD_NUMBER: builtins.int
    TARGET_WORLD_SPACE_POS_FIELD_NUMBER: builtins.int
    TARGET_UNIT_TAG_FIELD_NUMBER: builtins.int
    PROGRESS_FIELD_NUMBER: builtins.int
    ability_id: builtins.int
    @property
    def target_world_space_pos(self) -> s2clientprotocol.common_pb2.Point: ...
    target_unit_tag: builtins.int
    progress: builtins.float
    """Progress of train abilities. Range: [0.0, 1.0]"""
    def __init__(
        self,
        *,
        ability_id: builtins.int | None = ...,
        target_world_space_pos: s2clientprotocol.common_pb2.Point | None = ...,
        target_unit_tag: builtins.int | None = ...,
        progress: builtins.float | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["ability_id", b"ability_id", "progress", b"progress", "target", b"target", "target_unit_tag", b"target_unit_tag", "target_world_space_pos", b"target_world_space_pos"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["ability_id", b"ability_id", "progress", b"progress", "target", b"target", "target_unit_tag", b"target_unit_tag", "target_world_space_pos", b"target_world_space_pos"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["target", b"target"]) -> typing_extensions.Literal["target_world_space_pos", "target_unit_tag"] | None: ...

global___UnitOrder = UnitOrder

@typing_extensions.final
class PassengerUnit(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TAG_FIELD_NUMBER: builtins.int
    HEALTH_FIELD_NUMBER: builtins.int
    HEALTH_MAX_FIELD_NUMBER: builtins.int
    SHIELD_FIELD_NUMBER: builtins.int
    SHIELD_MAX_FIELD_NUMBER: builtins.int
    ENERGY_FIELD_NUMBER: builtins.int
    ENERGY_MAX_FIELD_NUMBER: builtins.int
    UNIT_TYPE_FIELD_NUMBER: builtins.int
    tag: builtins.int
    health: builtins.float
    health_max: builtins.float
    shield: builtins.float
    shield_max: builtins.float
    energy: builtins.float
    energy_max: builtins.float
    unit_type: builtins.int
    def __init__(
        self,
        *,
        tag: builtins.int | None = ...,
        health: builtins.float | None = ...,
        health_max: builtins.float | None = ...,
        shield: builtins.float | None = ...,
        shield_max: builtins.float | None = ...,
        energy: builtins.float | None = ...,
        energy_max: builtins.float | None = ...,
        unit_type: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["energy", b"energy", "energy_max", b"energy_max", "health", b"health", "health_max", b"health_max", "shield", b"shield", "shield_max", b"shield_max", "tag", b"tag", "unit_type", b"unit_type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["energy", b"energy", "energy_max", b"energy_max", "health", b"health", "health_max", b"health_max", "shield", b"shield", "shield_max", b"shield_max", "tag", b"tag", "unit_type", b"unit_type"]) -> None: ...

global___PassengerUnit = PassengerUnit

@typing_extensions.final
class RallyTarget(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    POINT_FIELD_NUMBER: builtins.int
    TAG_FIELD_NUMBER: builtins.int
    @property
    def point(self) -> s2clientprotocol.common_pb2.Point:
        """Will always be filled."""
    tag: builtins.int
    """Only if it's targeting a unit."""
    def __init__(
        self,
        *,
        point: s2clientprotocol.common_pb2.Point | None = ...,
        tag: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["point", b"point", "tag", b"tag"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["point", b"point", "tag", b"tag"]) -> None: ...

global___RallyTarget = RallyTarget

@typing_extensions.final
class Unit(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DISPLAY_TYPE_FIELD_NUMBER: builtins.int
    ALLIANCE_FIELD_NUMBER: builtins.int
    TAG_FIELD_NUMBER: builtins.int
    UNIT_TYPE_FIELD_NUMBER: builtins.int
    OWNER_FIELD_NUMBER: builtins.int
    POS_FIELD_NUMBER: builtins.int
    FACING_FIELD_NUMBER: builtins.int
    RADIUS_FIELD_NUMBER: builtins.int
    BUILD_PROGRESS_FIELD_NUMBER: builtins.int
    CLOAK_FIELD_NUMBER: builtins.int
    BUFF_IDS_FIELD_NUMBER: builtins.int
    DETECT_RANGE_FIELD_NUMBER: builtins.int
    RADAR_RANGE_FIELD_NUMBER: builtins.int
    IS_SELECTED_FIELD_NUMBER: builtins.int
    IS_ON_SCREEN_FIELD_NUMBER: builtins.int
    IS_BLIP_FIELD_NUMBER: builtins.int
    IS_POWERED_FIELD_NUMBER: builtins.int
    IS_ACTIVE_FIELD_NUMBER: builtins.int
    ATTACK_UPGRADE_LEVEL_FIELD_NUMBER: builtins.int
    ARMOR_UPGRADE_LEVEL_FIELD_NUMBER: builtins.int
    SHIELD_UPGRADE_LEVEL_FIELD_NUMBER: builtins.int
    HEALTH_FIELD_NUMBER: builtins.int
    HEALTH_MAX_FIELD_NUMBER: builtins.int
    SHIELD_FIELD_NUMBER: builtins.int
    SHIELD_MAX_FIELD_NUMBER: builtins.int
    ENERGY_FIELD_NUMBER: builtins.int
    ENERGY_MAX_FIELD_NUMBER: builtins.int
    MINERAL_CONTENTS_FIELD_NUMBER: builtins.int
    VESPENE_CONTENTS_FIELD_NUMBER: builtins.int
    IS_FLYING_FIELD_NUMBER: builtins.int
    IS_BURROWED_FIELD_NUMBER: builtins.int
    IS_HALLUCINATION_FIELD_NUMBER: builtins.int
    ORDERS_FIELD_NUMBER: builtins.int
    ADD_ON_TAG_FIELD_NUMBER: builtins.int
    PASSENGERS_FIELD_NUMBER: builtins.int
    CARGO_SPACE_TAKEN_FIELD_NUMBER: builtins.int
    CARGO_SPACE_MAX_FIELD_NUMBER: builtins.int
    ASSIGNED_HARVESTERS_FIELD_NUMBER: builtins.int
    IDEAL_HARVESTERS_FIELD_NUMBER: builtins.int
    WEAPON_COOLDOWN_FIELD_NUMBER: builtins.int
    ENGAGED_TARGET_TAG_FIELD_NUMBER: builtins.int
    BUFF_DURATION_REMAIN_FIELD_NUMBER: builtins.int
    BUFF_DURATION_MAX_FIELD_NUMBER: builtins.int
    RALLY_TARGETS_FIELD_NUMBER: builtins.int
    display_type: global___DisplayType.ValueType
    """Fields are populated based on type/alliance"""
    alliance: global___Alliance.ValueType
    tag: builtins.int
    """Unique identifier for a unit"""
    unit_type: builtins.int
    owner: builtins.int
    @property
    def pos(self) -> s2clientprotocol.common_pb2.Point: ...
    facing: builtins.float
    radius: builtins.float
    build_progress: builtins.float
    """Range: [0.0, 1.0]"""
    cloak: global___CloakState.ValueType
    @property
    def buff_ids(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    detect_range: builtins.float
    radar_range: builtins.float
    is_selected: builtins.bool
    is_on_screen: builtins.bool
    """Visible and within the camera frustrum."""
    is_blip: builtins.bool
    """Detected by sensor tower"""
    is_powered: builtins.bool
    is_active: builtins.bool
    """Building is training/researching (ie animated)."""
    attack_upgrade_level: builtins.int
    armor_upgrade_level: builtins.int
    shield_upgrade_level: builtins.int
    health: builtins.float
    """Not populated for snapshots"""
    health_max: builtins.float
    shield: builtins.float
    shield_max: builtins.float
    energy: builtins.float
    energy_max: builtins.float
    mineral_contents: builtins.int
    vespene_contents: builtins.int
    is_flying: builtins.bool
    is_burrowed: builtins.bool
    is_hallucination: builtins.bool
    """Unit is your own or detected as a hallucination."""
    @property
    def orders(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___UnitOrder]:
        """Not populated for enemies"""
    add_on_tag: builtins.int
    @property
    def passengers(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___PassengerUnit]: ...
    cargo_space_taken: builtins.int
    cargo_space_max: builtins.int
    assigned_harvesters: builtins.int
    ideal_harvesters: builtins.int
    weapon_cooldown: builtins.float
    engaged_target_tag: builtins.int
    buff_duration_remain: builtins.int
    """How long a buff or unit is still around (eg mule, broodling, chronoboost)."""
    buff_duration_max: builtins.int
    """How long the buff or unit is still around (eg mule, broodling, chronoboost)."""
    @property
    def rally_targets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___RallyTarget]: ...
    def __init__(
        self,
        *,
        display_type: global___DisplayType.ValueType | None = ...,
        alliance: global___Alliance.ValueType | None = ...,
        tag: builtins.int | None = ...,
        unit_type: builtins.int | None = ...,
        owner: builtins.int | None = ...,
        pos: s2clientprotocol.common_pb2.Point | None = ...,
        facing: builtins.float | None = ...,
        radius: builtins.float | None = ...,
        build_progress: builtins.float | None = ...,
        cloak: global___CloakState.ValueType | None = ...,
        buff_ids: collections.abc.Iterable[builtins.int] | None = ...,
        detect_range: builtins.float | None = ...,
        radar_range: builtins.float | None = ...,
        is_selected: builtins.bool | None = ...,
        is_on_screen: builtins.bool | None = ...,
        is_blip: builtins.bool | None = ...,
        is_powered: builtins.bool | None = ...,
        is_active: builtins.bool | None = ...,
        attack_upgrade_level: builtins.int | None = ...,
        armor_upgrade_level: builtins.int | None = ...,
        shield_upgrade_level: builtins.int | None = ...,
        health: builtins.float | None = ...,
        health_max: builtins.float | None = ...,
        shield: builtins.float | None = ...,
        shield_max: builtins.float | None = ...,
        energy: builtins.float | None = ...,
        energy_max: builtins.float | None = ...,
        mineral_contents: builtins.int | None = ...,
        vespene_contents: builtins.int | None = ...,
        is_flying: builtins.bool | None = ...,
        is_burrowed: builtins.bool | None = ...,
        is_hallucination: builtins.bool | None = ...,
        orders: collections.abc.Iterable[global___UnitOrder] | None = ...,
        add_on_tag: builtins.int | None = ...,
        passengers: collections.abc.Iterable[global___PassengerUnit] | None = ...,
        cargo_space_taken: builtins.int | None = ...,
        cargo_space_max: builtins.int | None = ...,
        assigned_harvesters: builtins.int | None = ...,
        ideal_harvesters: builtins.int | None = ...,
        weapon_cooldown: builtins.float | None = ...,
        engaged_target_tag: builtins.int | None = ...,
        buff_duration_remain: builtins.int | None = ...,
        buff_duration_max: builtins.int | None = ...,
        rally_targets: collections.abc.Iterable[global___RallyTarget] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["add_on_tag", b"add_on_tag", "alliance", b"alliance", "armor_upgrade_level", b"armor_upgrade_level", "assigned_harvesters", b"assigned_harvesters", "attack_upgrade_level", b"attack_upgrade_level", "buff_duration_max", b"buff_duration_max", "buff_duration_remain", b"buff_duration_remain", "build_progress", b"build_progress", "cargo_space_max", b"cargo_space_max", "cargo_space_taken", b"cargo_space_taken", "cloak", b"cloak", "detect_range", b"detect_range", "display_type", b"display_type", "energy", b"energy", "energy_max", b"energy_max", "engaged_target_tag", b"engaged_target_tag", "facing", b"facing", "health", b"health", "health_max", b"health_max", "ideal_harvesters", b"ideal_harvesters", "is_active", b"is_active", "is_blip", b"is_blip", "is_burrowed", b"is_burrowed", "is_flying", b"is_flying", "is_hallucination", b"is_hallucination", "is_on_screen", b"is_on_screen", "is_powered", b"is_powered", "is_selected", b"is_selected", "mineral_contents", b"mineral_contents", "owner", b"owner", "pos", b"pos", "radar_range", b"radar_range", "radius", b"radius", "shield", b"shield", "shield_max", b"shield_max", "shield_upgrade_level", b"shield_upgrade_level", "tag", b"tag", "unit_type", b"unit_type", "vespene_contents", b"vespene_contents", "weapon_cooldown", b"weapon_cooldown"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["add_on_tag", b"add_on_tag", "alliance", b"alliance", "armor_upgrade_level", b"armor_upgrade_level", "assigned_harvesters", b"assigned_harvesters", "attack_upgrade_level", b"attack_upgrade_level", "buff_duration_max", b"buff_duration_max", "buff_duration_remain", b"buff_duration_remain", "buff_ids", b"buff_ids", "build_progress", b"build_progress", "cargo_space_max", b"cargo_space_max", "cargo_space_taken", b"cargo_space_taken", "cloak", b"cloak", "detect_range", b"detect_range", "display_type", b"display_type", "energy", b"energy", "energy_max", b"energy_max", "engaged_target_tag", b"engaged_target_tag", "facing", b"facing", "health", b"health", "health_max", b"health_max", "ideal_harvesters", b"ideal_harvesters", "is_active", b"is_active", "is_blip", b"is_blip", "is_burrowed", b"is_burrowed", "is_flying", b"is_flying", "is_hallucination", b"is_hallucination", "is_on_screen", b"is_on_screen", "is_powered", b"is_powered", "is_selected", b"is_selected", "mineral_contents", b"mineral_contents", "orders", b"orders", "owner", b"owner", "passengers", b"passengers", "pos", b"pos", "radar_range", b"radar_range", "radius", b"radius", "rally_targets", b"rally_targets", "shield", b"shield", "shield_max", b"shield_max", "shield_upgrade_level", b"shield_upgrade_level", "tag", b"tag", "unit_type", b"unit_type", "vespene_contents", b"vespene_contents", "weapon_cooldown", b"weapon_cooldown"]) -> None: ...

global___Unit = Unit

@typing_extensions.final
class MapState(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VISIBILITY_FIELD_NUMBER: builtins.int
    CREEP_FIELD_NUMBER: builtins.int
    @property
    def visibility(self) -> s2clientprotocol.common_pb2.ImageData:
        """1 byte visibility layer."""
    @property
    def creep(self) -> s2clientprotocol.common_pb2.ImageData:
        """1 bit creep layer."""
    def __init__(
        self,
        *,
        visibility: s2clientprotocol.common_pb2.ImageData | None = ...,
        creep: s2clientprotocol.common_pb2.ImageData | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["creep", b"creep", "visibility", b"visibility"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["creep", b"creep", "visibility", b"visibility"]) -> None: ...

global___MapState = MapState

@typing_extensions.final
class Event(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DEAD_UNITS_FIELD_NUMBER: builtins.int
    @property
    def dead_units(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        dead_units: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["dead_units", b"dead_units"]) -> None: ...

global___Event = Event

@typing_extensions.final
class Effect(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EFFECT_ID_FIELD_NUMBER: builtins.int
    POS_FIELD_NUMBER: builtins.int
    ALLIANCE_FIELD_NUMBER: builtins.int
    OWNER_FIELD_NUMBER: builtins.int
    RADIUS_FIELD_NUMBER: builtins.int
    effect_id: builtins.int
    @property
    def pos(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[s2clientprotocol.common_pb2.Point2D]:
        """Effect may impact multiple locations. (eg. Lurker attack)"""
    alliance: global___Alliance.ValueType
    owner: builtins.int
    radius: builtins.float
    def __init__(
        self,
        *,
        effect_id: builtins.int | None = ...,
        pos: collections.abc.Iterable[s2clientprotocol.common_pb2.Point2D] | None = ...,
        alliance: global___Alliance.ValueType | None = ...,
        owner: builtins.int | None = ...,
        radius: builtins.float | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["alliance", b"alliance", "effect_id", b"effect_id", "owner", b"owner", "radius", b"radius"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["alliance", b"alliance", "effect_id", b"effect_id", "owner", b"owner", "pos", b"pos", "radius", b"radius"]) -> None: ...

global___Effect = Effect

@typing_extensions.final
class ActionRaw(google.protobuf.message.Message):
    """
    Action
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UNIT_COMMAND_FIELD_NUMBER: builtins.int
    CAMERA_MOVE_FIELD_NUMBER: builtins.int
    TOGGLE_AUTOCAST_FIELD_NUMBER: builtins.int
    @property
    def unit_command(self) -> global___ActionRawUnitCommand: ...
    @property
    def camera_move(self) -> global___ActionRawCameraMove: ...
    @property
    def toggle_autocast(self) -> global___ActionRawToggleAutocast: ...
    def __init__(
        self,
        *,
        unit_command: global___ActionRawUnitCommand | None = ...,
        camera_move: global___ActionRawCameraMove | None = ...,
        toggle_autocast: global___ActionRawToggleAutocast | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["action", b"action", "camera_move", b"camera_move", "toggle_autocast", b"toggle_autocast", "unit_command", b"unit_command"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["action", b"action", "camera_move", b"camera_move", "toggle_autocast", b"toggle_autocast", "unit_command", b"unit_command"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["action", b"action"]) -> typing_extensions.Literal["unit_command", "camera_move", "toggle_autocast"] | None: ...

global___ActionRaw = ActionRaw

@typing_extensions.final
class ActionRawUnitCommand(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ABILITY_ID_FIELD_NUMBER: builtins.int
    TARGET_WORLD_SPACE_POS_FIELD_NUMBER: builtins.int
    TARGET_UNIT_TAG_FIELD_NUMBER: builtins.int
    UNIT_TAGS_FIELD_NUMBER: builtins.int
    QUEUE_COMMAND_FIELD_NUMBER: builtins.int
    ability_id: builtins.int
    @property
    def target_world_space_pos(self) -> s2clientprotocol.common_pb2.Point2D: ...
    target_unit_tag: builtins.int
    @property
    def unit_tags(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    queue_command: builtins.bool
    def __init__(
        self,
        *,
        ability_id: builtins.int | None = ...,
        target_world_space_pos: s2clientprotocol.common_pb2.Point2D | None = ...,
        target_unit_tag: builtins.int | None = ...,
        unit_tags: collections.abc.Iterable[builtins.int] | None = ...,
        queue_command: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["ability_id", b"ability_id", "queue_command", b"queue_command", "target", b"target", "target_unit_tag", b"target_unit_tag", "target_world_space_pos", b"target_world_space_pos"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["ability_id", b"ability_id", "queue_command", b"queue_command", "target", b"target", "target_unit_tag", b"target_unit_tag", "target_world_space_pos", b"target_world_space_pos", "unit_tags", b"unit_tags"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["target", b"target"]) -> typing_extensions.Literal["target_world_space_pos", "target_unit_tag"] | None: ...

global___ActionRawUnitCommand = ActionRawUnitCommand

@typing_extensions.final
class ActionRawCameraMove(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CENTER_WORLD_SPACE_FIELD_NUMBER: builtins.int
    @property
    def center_world_space(self) -> s2clientprotocol.common_pb2.Point: ...
    def __init__(
        self,
        *,
        center_world_space: s2clientprotocol.common_pb2.Point | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["center_world_space", b"center_world_space"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["center_world_space", b"center_world_space"]) -> None: ...

global___ActionRawCameraMove = ActionRawCameraMove

@typing_extensions.final
class ActionRawToggleAutocast(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ABILITY_ID_FIELD_NUMBER: builtins.int
    UNIT_TAGS_FIELD_NUMBER: builtins.int
    ability_id: builtins.int
    @property
    def unit_tags(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        ability_id: builtins.int | None = ...,
        unit_tags: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["ability_id", b"ability_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["ability_id", b"ability_id", "unit_tags", b"unit_tags"]) -> None: ...

global___ActionRawToggleAutocast = ActionRawToggleAutocast



from _typeshed import Incomplete
from typing import Any

class Deleted:
    key: Any
    version_id: Any
    delete_marker: Any
    delete_marker_version_id: Any
    def __init__(
        self,
        key: Incomplete | None = None,
        version_id: Incomplete | None = None,
        delete_marker: bool = False,
        delete_marker_version_id: Incomplete | None = None,
    ) -> None: ...
    def startElement(self, name, attrs, connection): ...
    def endElement(self, name, value, connection): ...

class Error:
    key: Any
    version_id: Any
    code: Any
    message: Any
    def __init__(
        self,
        key: Incomplete | None = None,
        version_id: Incomplete | None = None,
        code: Incomplete | None = None,
        message: Incomplete | None = None,
    ) -> None: ...
    def startElement(self, name, attrs, connection): ...
    def endElement(self, name, value, connection): ...

class MultiDeleteResult:
    bucket: Any
    deleted: Any
    errors: Any
    def __init__(self, bucket: Incomplete | None = None) -> None: ...
    def startElement(self, name, attrs, connection): ...
    def endElement(self, name, value, connection): ...



from _typeshed import Incomplete
from types import TracebackType

from .models.segment import SegmentContextManager
from .models.subsegment import SubsegmentContextManager
from .recorder import AWSXRayRecorder

class AsyncSegmentContextManager(SegmentContextManager):
    async def __aenter__(self): ...
    async def __aexit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...

class AsyncSubsegmentContextManager(SubsegmentContextManager):
    async def __call__(self, wrapped, instance, args, kwargs): ...
    async def __aenter__(self): ...
    async def __aexit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...

class AsyncAWSXRayRecorder(AWSXRayRecorder):
    def capture_async(self, name: Incomplete | None = None): ...
    def in_segment_async(self, name: Incomplete | None = None, **segment_kwargs): ...
    def in_subsegment_async(self, name: Incomplete | None = None, **subsegment_kwargs): ...
    async def record_subsegment_async(self, wrapped, instance, args, kwargs, name, namespace, meta_processor): ...



import sys
from types import TracebackType
from typing import Any, Literal
from typing_extensions import Self

from ._distutils.errors import DistutilsError

__all__ = ["AbstractSandbox", "DirectorySandbox", "SandboxViolation", "run_setup"]

class UnpickleableException(Exception):
    @staticmethod
    def dump(type, exc): ...

class ExceptionSaver:
    def __enter__(self) -> Self: ...
    def __exit__(
        self, type: type[BaseException] | None, exc: BaseException | None, tb: TracebackType | None
    ) -> Literal[True] | None: ...
    def resume(self) -> None: ...

def run_setup(setup_script, args): ...

class AbstractSandbox:
    def __enter__(self) -> None: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: TracebackType | None
    ) -> None: ...
    def run(self, func): ...
    # Dynamically created
    if sys.platform == "win32":
        def startfile(self, path, *args, **kw): ...
    else:
        def chown(self, path, *args, **kw): ...
        def chroot(self, path, *args, **kw): ...
        def lchown(self, path, *args, **kw): ...
        def mkfifo(self, path, *args, **kw): ...
        def mknod(self, path, *args, **kw): ...
        def pathconf(self, path, *args, **kw): ...

    def access(self, path, *args, **kw): ...
    def chdir(self, path, *args, **kw): ...
    def chmod(self, path, *args, **kw): ...
    def getcwd(self, *args, **kw): ...
    def link(self, src, dst, *args, **kw): ...
    def listdir(self, path, *args, **kw): ...
    def lstat(self, path, *args, **kw): ...
    def mkdir(self, path, *args, **kw): ...
    def open(self, path, *args, **kw): ...
    def readlink(self, path, *args, **kw): ...
    def remove(self, path, *args, **kw): ...
    def rename(self, src, dst, *args, **kw): ...
    def rmdir(self, path, *args, **kw): ...
    def stat(self, path, *args, **kw): ...
    def symlink(self, src, dst, *args, **kw): ...
    def unlink(self, path, *args, **kw): ...
    def utime(self, path, *args, **kw): ...

class DirectorySandbox(AbstractSandbox):
    write_ops: Any
    def __init__(self, sandbox, exceptions=...) -> None: ...
    def tmpnam(self) -> None: ...
    def open(self, file, flags, mode: int = 511, *args, **kw): ...  # type: ignore[override]

class SandboxViolation(DistutilsError):
    tmpl: Any



from _typeshed import Incomplete
from typing import Any

class OAuth1Error(Exception):
    error: Any
    description: str
    uri: Any
    status_code: Any
    def __init__(
        self,
        description: Incomplete | None = None,
        uri: Incomplete | None = None,
        status_code: int = 400,
        request: Incomplete | None = None,
    ) -> None: ...
    def in_uri(self, uri): ...
    @property
    def twotuples(self): ...
    @property
    def urlencoded(self): ...

class InsecureTransportError(OAuth1Error):
    error: str
    description: str

class InvalidSignatureMethodError(OAuth1Error):
    error: str

class InvalidRequestError(OAuth1Error):
    error: str

class InvalidClientError(OAuth1Error):
    error: str



from networkx.utils.backends import _dispatch

@_dispatch
def karate_club_graph(): ...
@_dispatch
def davis_southern_women_graph(): ...
@_dispatch
def florentine_families_graph(): ...
@_dispatch
def les_miserables_graph(): ...



from _typeshed import SupportsItemAccess
from datetime import datetime, timedelta
from typing import Any

from wtforms.csrf.core import CSRF, CSRFTokenField
from wtforms.form import BaseForm
from wtforms.meta import DefaultMeta

class SessionCSRF(CSRF):
    TIME_FORMAT: str
    form_meta: DefaultMeta
    def generate_csrf_token(self, csrf_token_field: CSRFTokenField) -> str: ...
    def validate_csrf_token(self, form: BaseForm, field: CSRFTokenField) -> None: ...
    def now(self) -> datetime: ...
    @property
    def time_limit(self) -> timedelta: ...
    @property
    def session(self) -> SupportsItemAccess[str, Any]: ...



from _typeshed import Incomplete
from typing import NoReturn
from typing_extensions import Self

from stripe.api_resources import ApplicationFee as ApplicationFee
from stripe.api_resources.abstract import UpdateableAPIResource as UpdateableAPIResource

class ApplicationFeeRefund(UpdateableAPIResource):
    OBJECT_NAME: str
    @classmethod
    def modify(cls, fee, sid: str, **params) -> Self: ...  # type: ignore[override]
    def instance_url(self) -> str: ...
    @classmethod
    def retrieve(cls, id, api_key: Incomplete | None = None, **params) -> NoReturn: ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def closeness_vitality(
    G, node: Incomplete | None = None, weight: Incomplete | None = None, wiener_index: Incomplete | None = None
): ...



from _typeshed import Incomplete
from collections.abc import Generator
from typing import ClassVar

from ..core import IslamoWesternCalendar

class Kenya(IslamoWesternCalendar):
    include_labour_day: ClassVar[bool]
    include_good_friday: ClassVar[bool]
    include_easter_monday: ClassVar[bool]
    include_eid_al_fitr: ClassVar[bool]
    include_day_of_sacrifice: ClassVar[bool]
    shift_sunday_holidays: ClassVar[bool]
    WEEKEND_DAYS: Incomplete
    FIXED_HOLIDAYS: Incomplete
    def get_fixed_holidays(self, year): ...
    def get_shifted_holidays(self, dates) -> Generator[Incomplete, None, None]: ...
    def get_calendar_holidays(self, year): ...



from _typeshed import Incomplete

class MeasurementSchemaList:
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(self, measurement_schemas: Incomplete | None = None) -> None: ...
    @property
    def measurement_schemas(self): ...
    @measurement_schemas.setter
    def measurement_schemas(self, measurement_schemas) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from . import coreviews as coreviews, filters as filters, graphviews as graphviews, reportviews as reportviews
from .digraph import DiGraph as DiGraph
from .function import *
from .graph import Graph as Graph
from .graphviews import reverse_view as reverse_view, subgraph_view as subgraph_view
from .multidigraph import MultiDiGraph as MultiDiGraph
from .multigraph import MultiGraph as MultiGraph



from _typeshed import Incomplete

class ExchangeRateQuoteRequest:
    quotes: Incomplete
    def __init__(self) -> None: ...
    def add_exchange_rate_quote_input(self, attributes): ...
    def to_graphql_variables(self): ...



from _typeshed import SupportsItems, SupportsKeysAndGetItem
from _typeshed.wsgi import StartResponse, WSGIApplication, WSGIEnvironment
from collections.abc import Iterable
from string import Template
from typing import Any, Literal, Protocol
from typing_extensions import Self

from webob.response import Response

class _JSONFormatter(Protocol):
    def __call__(self, *, body: str, status: str, title: str, environ: WSGIEnvironment) -> str: ...

class HTTPException(Exception):
    wsgi_response: Response
    def __init__(self, message: str, wsgi_response: Response) -> None: ...
    def __call__(self, environ: WSGIEnvironment, start_response: StartResponse) -> Iterable[bytes]: ...

class WSGIHTTPException(Response, HTTPException):
    code: int
    title: str
    explanation: str
    body_template_obj: Template
    plain_template_obj: Template
    html_template_obj: Template
    empty_body: bool
    detail: str | None
    comment: str | None
    def __init__(
        self,
        detail: str | None = None,
        headers: SupportsItems[str, str] | SupportsKeysAndGetItem[str, str] | Iterable[tuple[str, str]] | None = None,
        comment: str | None = None,
        body_template: str | None = None,
        json_formatter: _JSONFormatter | None = None,
        **kw: Any,
    ) -> None: ...
    def plain_body(self, environ: WSGIEnvironment) -> str: ...
    def html_body(self, environ: WSGIEnvironment) -> str: ...
    def json_formatter(self, body: str, status: str, title: str, environ: WSGIEnvironment) -> str: ...
    def json_body(self, environ: WSGIEnvironment) -> str: ...
    def generate_response(self, environ: WSGIEnvironment, start_response: StartResponse) -> Iterable[bytes]: ...
    @property
    def wsgi_response(self) -> Self: ...  # type: ignore[override]
    def __str__(self) -> str: ...  # type: ignore[override]  # noqa: Y029

class HTTPError(WSGIHTTPException): ...
class HTTPRedirection(WSGIHTTPException): ...
class HTTPOk(WSGIHTTPException): ...
class HTTPCreated(HTTPOk): ...
class HTTPAccepted(HTTPOk): ...
class HTTPNonAuthoritativeInformation(HTTPOk): ...

class HTTPNoContent(HTTPOk):
    empty_body: Literal[True]

class HTTPResetContent(HTTPOk):
    empty_body: Literal[True]

class HTTPPartialContent(HTTPOk): ...

class _HTTPMove(HTTPRedirection):
    explanation: str
    add_slash: bool
    def __init__(
        self,
        detail: str | None = None,
        headers: str | None = None,
        comment: str | None = None,
        body_template: str | None = None,
        location: str | None = None,
        add_slash: bool = False,
    ) -> None: ...

class HTTPMultipleChoices(_HTTPMove): ...
class HTTPMovedPermanently(_HTTPMove): ...
class HTTPFound(_HTTPMove): ...
class HTTPSeeOther(_HTTPMove): ...

class HTTPNotModified(HTTPRedirection):
    empty_body: Literal[True]

class HTTPUseProxy(_HTTPMove): ...
class HTTPTemporaryRedirect(_HTTPMove): ...
class HTTPPermanentRedirect(_HTTPMove): ...
class HTTPClientError(HTTPError): ...
class HTTPBadRequest(HTTPClientError): ...
class HTTPUnauthorized(HTTPClientError): ...
class HTTPPaymentRequired(HTTPClientError): ...
class HTTPForbidden(HTTPClientError): ...
class HTTPNotFound(HTTPClientError): ...
class HTTPMethodNotAllowed(HTTPClientError): ...
class HTTPNotAcceptable(HTTPClientError): ...
class HTTPProxyAuthenticationRequired(HTTPClientError): ...
class HTTPRequestTimeout(HTTPClientError): ...
class HTTPConflict(HTTPClientError): ...
class HTTPGone(HTTPClientError): ...
class HTTPLengthRequired(HTTPClientError): ...
class HTTPPreconditionFailed(HTTPClientError): ...
class HTTPRequestEntityTooLarge(HTTPClientError): ...
class HTTPRequestURITooLong(HTTPClientError): ...
class HTTPUnsupportedMediaType(HTTPClientError): ...
class HTTPRequestRangeNotSatisfiable(HTTPClientError): ...
class HTTPExpectationFailed(HTTPClientError): ...
class HTTPUnprocessableEntity(HTTPClientError): ...
class HTTPLocked(HTTPClientError): ...
class HTTPFailedDependency(HTTPClientError): ...
class HTTPPreconditionRequired(HTTPClientError): ...
class HTTPTooManyRequests(HTTPClientError): ...
class HTTPRequestHeaderFieldsTooLarge(HTTPClientError): ...
class HTTPUnavailableForLegalReasons(HTTPClientError): ...
class HTTPServerError(HTTPError): ...
class HTTPInternalServerError(HTTPServerError): ...
class HTTPNotImplemented(HTTPServerError): ...
class HTTPBadGateway(HTTPServerError): ...
class HTTPServiceUnavailable(HTTPServerError): ...
class HTTPGatewayTimeout(HTTPServerError): ...
class HTTPVersionNotSupported(HTTPServerError): ...
class HTTPInsufficientStorage(HTTPServerError): ...
class HTTPNetworkAuthenticationRequired(HTTPServerError): ...

class HTTPExceptionMiddleware:
    application: WSGIApplication
    def __init__(self, application: WSGIApplication) -> None: ...
    def __call__(self, environ: WSGIEnvironment, start_response: StartResponse) -> Iterable[bytes]: ...

status_map: dict[int, WSGIHTTPException]



from win32.lib.pywintypes import error as error

def GetHandle(*args): ...  # incomplete
def GetTracer(*args): ...  # incomplete
def InitRead(*args): ...  # incomplete
def InitWrite(*args): ...  # incomplete
def TermRead(*args): ...  # incomplete
def TermWrite(*args): ...  # incomplete
def blockingread(*args): ...  # incomplete
def flush(*args): ...  # incomplete
def read(*args): ...  # incomplete
def setprint(*args): ...  # incomplete
def write(*args): ...  # incomplete



__all__ = ["Version", "InvalidVersion", "VERSION_PATTERN"]

class InvalidVersion(ValueError): ...

class _BaseVersion:
    def __hash__(self) -> int: ...
    def __lt__(self, other: _BaseVersion) -> bool: ...
    def __le__(self, other: _BaseVersion) -> bool: ...
    def __eq__(self, other: object) -> bool: ...
    def __ge__(self, other: _BaseVersion) -> bool: ...
    def __gt__(self, other: _BaseVersion) -> bool: ...
    def __ne__(self, other: object) -> bool: ...

VERSION_PATTERN: str

class Version(_BaseVersion):
    def __init__(self, version: str) -> None: ...
    @property
    def epoch(self) -> int: ...
    @property
    def release(self) -> tuple[int, ...]: ...
    @property
    def pre(self) -> tuple[str, int] | None: ...
    @property
    def post(self) -> int | None: ...
    @property
    def dev(self) -> int | None: ...
    @property
    def local(self) -> str | None: ...
    @property
    def public(self) -> str: ...
    @property
    def base_version(self) -> str: ...
    @property
    def is_prerelease(self) -> bool: ...
    @property
    def is_postrelease(self) -> bool: ...
    @property
    def is_devrelease(self) -> bool: ...
    @property
    def major(self) -> int: ...
    @property
    def minor(self) -> int: ...
    @property
    def micro(self) -> int: ...



from _typeshed import Unused
from typing import Final

from Xlib.display import Display
from Xlib.protocol import rq
from Xlib.xobject import resource

extname: Final = "SECURITY"
SecurityClientTrusted: Final = 0
SecurityClientUntrusted: Final = 1
SecurityAuthorizationRevokedMask: Final = 1
AUTHID = rq.Card32

class QueryVersion(rq.ReplyRequest): ...

def query_version(self: Display | resource.Resource) -> QueryVersion: ...

class SecurityGenerateAuthorization(rq.ReplyRequest): ...

def generate_authorization(
    self: Display | resource.Resource,
    auth_proto: str,
    auth_data: bytes | bytearray = b"",
    timeout: int | None = None,
    trust_level: int | None = None,
    group: int | None = None,
    event_mask: int | None = None,
) -> SecurityGenerateAuthorization: ...

class SecurityRevokeAuthorization(rq.Request): ...

def revoke_authorization(self: Display | resource.Resource, authid: int) -> SecurityRevokeAuthorization: ...
def init(disp: Display, info: Unused) -> None: ...



from networkx.utils.backends import _dispatch

@_dispatch
def adjacency_data(G, attrs={"id": "id", "key": "key"}): ...
@_dispatch
def adjacency_graph(data, directed: bool = False, multigraph: bool = True, attrs={"id": "id", "key": "key"}): ...



from _typeshed import Incomplete
from typing import ClassVar

from ..core import WesternCalendar

class Paraguay(WesternCalendar):
    FIXED_HOLIDAYS: Incomplete
    include_labour_day: ClassVar[bool]
    include_holy_thursday: ClassVar[bool]
    include_good_friday: ClassVar[bool]
    include_easter_saturday: ClassVar[bool]
    include_immaculate_conception: ClassVar[bool]
    immaculate_conception_label: ClassVar[str]
    def get_heroes_day(self, year): ...
    def get_founding_of_asuncion(self, year): ...
    def get_boqueron_battle_victory_day(self, year): ...
    def get_fixed_holidays(self, year): ...



from typing import ClassVar, Literal

from .ImageFile import StubImageFile

def register_handler(handler) -> None: ...

class BufrStubImageFile(StubImageFile):
    format: ClassVar[Literal["BUFR"]]
    format_description: ClassVar[str]



from typing import Final

NoValue: Final = 0x0000
XValue: Final = 0x0001
YValue: Final = 0x0002
WidthValue: Final = 0x0004
HeightValue: Final = 0x0008
AllValues: Final = 0x000F
XNegative: Final = 0x0010
YNegative: Final = 0x0020
USPosition: Final = 0x001
USSize: Final = 0x002
PPosition: Final = 0x004
PSize: Final = 0x008
PMinSize: Final = 0x010
PMaxSize: Final = 0x020
PResizeInc: Final = 0x040
PAspect: Final = 0x080
PBaseSize: Final = 0x100
PWinGravity: Final = 0x200
PAllHints: Final = 252
InputHint: Final = 0x001
StateHint: Final = 0x002
IconPixmapHint: Final = 0x004
IconWindowHint: Final = 0x008
IconPositionHint: Final = 0x010
IconMaskHint: Final = 0x020
WindowGroupHint: Final = 0x040
MessageHint: Final = 0x080
UrgencyHint: Final = 0x100
AllHints: Final = 511
WithdrawnState: Final = 0
NormalState: Final = 1
IconicState: Final = 3
DontCareState: Final = 0
ZoomState: Final = 2
InactiveState: Final = 4
RectangleOut: Final = 0
RectangleIn: Final = 1
RectanglePart: Final = 2
VisualNoMask: Final = 0x0
VisualIDMask: Final = 0x1
VisualScreenMask: Final = 0x2
VisualDepthMask: Final = 0x4
VisualClassMask: Final = 0x8
VisualRedMaskMask: Final = 0x10
VisualGreenMaskMask: Final = 0x20
VisualBlueMaskMask: Final = 0x40
VisualColormapSizeMask: Final = 0x80
VisualBitsPerRGBMask: Final = 0x100
VisualAllMask: Final = 0x1FF
ReleaseByFreeingColormap: Final = 1
BitmapSuccess: Final = 0
BitmapOpenFailed: Final = 1
BitmapFileInvalid: Final = 2
BitmapNoMemory: Final = 3
XCSUCCESS: Final = 0
XCNOMEM: Final = 1
XCNOENT: Final = 2



from markdown.core import Markdown

from . import util

def build_postprocessors(md: Markdown, **kwargs) -> util.Registry[Postprocessor]: ...

class Postprocessor(util.Processor):
    def run(self, text: str) -> str: ...

class RawHtmlPostprocessor(Postprocessor):
    def isblocklevel(self, html: str) -> bool: ...
    def stash_to_string(self, text: str) -> str: ...

class AndSubstitutePostprocessor(Postprocessor): ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import typing as typing_extensions

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import s2clientprotocol.common_pb2
import s2clientprotocol.error_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class RequestQuery(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PATHING_FIELD_NUMBER: builtins.int
    ABILITIES_FIELD_NUMBER: builtins.int
    PLACEMENTS_FIELD_NUMBER: builtins.int
    IGNORE_RESOURCE_REQUIREMENTS_FIELD_NUMBER: builtins.int
    @property
    def pathing(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___RequestQueryPathing]: ...
    @property
    def abilities(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___RequestQueryAvailableAbilities]: ...
    @property
    def placements(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___RequestQueryBuildingPlacement]: ...
    ignore_resource_requirements: builtins.bool
    """Ignores requirements like food, minerals and so on."""
    def __init__(
        self,
        *,
        pathing: collections.abc.Iterable[global___RequestQueryPathing] | None = ...,
        abilities: collections.abc.Iterable[global___RequestQueryAvailableAbilities] | None = ...,
        placements: collections.abc.Iterable[global___RequestQueryBuildingPlacement] | None = ...,
        ignore_resource_requirements: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["ignore_resource_requirements", b"ignore_resource_requirements"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["abilities", b"abilities", "ignore_resource_requirements", b"ignore_resource_requirements", "pathing", b"pathing", "placements", b"placements"]) -> None: ...

global___RequestQuery = RequestQuery

@typing_extensions.final
class ResponseQuery(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PATHING_FIELD_NUMBER: builtins.int
    ABILITIES_FIELD_NUMBER: builtins.int
    PLACEMENTS_FIELD_NUMBER: builtins.int
    @property
    def pathing(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ResponseQueryPathing]: ...
    @property
    def abilities(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ResponseQueryAvailableAbilities]: ...
    @property
    def placements(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ResponseQueryBuildingPlacement]: ...
    def __init__(
        self,
        *,
        pathing: collections.abc.Iterable[global___ResponseQueryPathing] | None = ...,
        abilities: collections.abc.Iterable[global___ResponseQueryAvailableAbilities] | None = ...,
        placements: collections.abc.Iterable[global___ResponseQueryBuildingPlacement] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["abilities", b"abilities", "pathing", b"pathing", "placements", b"placements"]) -> None: ...

global___ResponseQuery = ResponseQuery

@typing_extensions.final
class RequestQueryPathing(google.protobuf.message.Message):
    """--------------------------------------------------------------------------------------------------"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    START_POS_FIELD_NUMBER: builtins.int
    UNIT_TAG_FIELD_NUMBER: builtins.int
    END_POS_FIELD_NUMBER: builtins.int
    @property
    def start_pos(self) -> s2clientprotocol.common_pb2.Point2D: ...
    unit_tag: builtins.int
    @property
    def end_pos(self) -> s2clientprotocol.common_pb2.Point2D: ...
    def __init__(
        self,
        *,
        start_pos: s2clientprotocol.common_pb2.Point2D | None = ...,
        unit_tag: builtins.int | None = ...,
        end_pos: s2clientprotocol.common_pb2.Point2D | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["end_pos", b"end_pos", "start", b"start", "start_pos", b"start_pos", "unit_tag", b"unit_tag"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["end_pos", b"end_pos", "start", b"start", "start_pos", b"start_pos", "unit_tag", b"unit_tag"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["start", b"start"]) -> typing_extensions.Literal["start_pos", "unit_tag"] | None: ...

global___RequestQueryPathing = RequestQueryPathing

@typing_extensions.final
class ResponseQueryPathing(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DISTANCE_FIELD_NUMBER: builtins.int
    distance: builtins.float
    """0 if no path exists"""
    def __init__(
        self,
        *,
        distance: builtins.float | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["distance", b"distance"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["distance", b"distance"]) -> None: ...

global___ResponseQueryPathing = ResponseQueryPathing

@typing_extensions.final
class RequestQueryAvailableAbilities(google.protobuf.message.Message):
    """--------------------------------------------------------------------------------------------------"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UNIT_TAG_FIELD_NUMBER: builtins.int
    unit_tag: builtins.int
    def __init__(
        self,
        *,
        unit_tag: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["unit_tag", b"unit_tag"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["unit_tag", b"unit_tag"]) -> None: ...

global___RequestQueryAvailableAbilities = RequestQueryAvailableAbilities

@typing_extensions.final
class ResponseQueryAvailableAbilities(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ABILITIES_FIELD_NUMBER: builtins.int
    UNIT_TAG_FIELD_NUMBER: builtins.int
    UNIT_TYPE_ID_FIELD_NUMBER: builtins.int
    @property
    def abilities(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[s2clientprotocol.common_pb2.AvailableAbility]: ...
    unit_tag: builtins.int
    unit_type_id: builtins.int
    def __init__(
        self,
        *,
        abilities: collections.abc.Iterable[s2clientprotocol.common_pb2.AvailableAbility] | None = ...,
        unit_tag: builtins.int | None = ...,
        unit_type_id: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["unit_tag", b"unit_tag", "unit_type_id", b"unit_type_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["abilities", b"abilities", "unit_tag", b"unit_tag", "unit_type_id", b"unit_type_id"]) -> None: ...

global___ResponseQueryAvailableAbilities = ResponseQueryAvailableAbilities

@typing_extensions.final
class RequestQueryBuildingPlacement(google.protobuf.message.Message):
    """--------------------------------------------------------------------------------------------------"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ABILITY_ID_FIELD_NUMBER: builtins.int
    TARGET_POS_FIELD_NUMBER: builtins.int
    PLACING_UNIT_TAG_FIELD_NUMBER: builtins.int
    ability_id: builtins.int
    @property
    def target_pos(self) -> s2clientprotocol.common_pb2.Point2D: ...
    placing_unit_tag: builtins.int
    """Not required"""
    def __init__(
        self,
        *,
        ability_id: builtins.int | None = ...,
        target_pos: s2clientprotocol.common_pb2.Point2D | None = ...,
        placing_unit_tag: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["ability_id", b"ability_id", "placing_unit_tag", b"placing_unit_tag", "target_pos", b"target_pos"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["ability_id", b"ability_id", "placing_unit_tag", b"placing_unit_tag", "target_pos", b"target_pos"]) -> None: ...

global___RequestQueryBuildingPlacement = RequestQueryBuildingPlacement

@typing_extensions.final
class ResponseQueryBuildingPlacement(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESULT_FIELD_NUMBER: builtins.int
    result: s2clientprotocol.error_pb2.ActionResult.ValueType
    def __init__(
        self,
        *,
        result: s2clientprotocol.error_pb2.ActionResult.ValueType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["result", b"result"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["result", b"result"]) -> None: ...

global___ResponseQueryBuildingPlacement = ResponseQueryBuildingPlacement



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def latapy_clustering(G, nodes: Incomplete | None = None, mode: str = "dot"): ...

clustering = latapy_clustering

@_dispatch
def average_clustering(G, nodes: Incomplete | None = None, mode: str = "dot"): ...
@_dispatch
def robins_alexander_clustering(G): ...



from typing import Final

XK_space: Final = 0x020
XK_exclam: Final = 0x021
XK_quotedbl: Final = 0x022
XK_numbersign: Final = 0x023
XK_dollar: Final = 0x024
XK_percent: Final = 0x025
XK_ampersand: Final = 0x026
XK_apostrophe: Final = 0x027
XK_quoteright: Final = 0x027
XK_parenleft: Final = 0x028
XK_parenright: Final = 0x029
XK_asterisk: Final = 0x02A
XK_plus: Final = 0x02B
XK_comma: Final = 0x02C
XK_minus: Final = 0x02D
XK_period: Final = 0x02E
XK_slash: Final = 0x02F
XK_0: Final = 0x030
XK_1: Final = 0x031
XK_2: Final = 0x032
XK_3: Final = 0x033
XK_4: Final = 0x034
XK_5: Final = 0x035
XK_6: Final = 0x036
XK_7: Final = 0x037
XK_8: Final = 0x038
XK_9: Final = 0x039
XK_colon: Final = 0x03A
XK_semicolon: Final = 0x03B
XK_less: Final = 0x03C
XK_equal: Final = 0x03D
XK_greater: Final = 0x03E
XK_question: Final = 0x03F
XK_at: Final = 0x040
XK_A: Final = 0x041
XK_B: Final = 0x042
XK_C: Final = 0x043
XK_D: Final = 0x044
XK_E: Final = 0x045
XK_F: Final = 0x046
XK_G: Final = 0x047
XK_H: Final = 0x048
XK_I: Final = 0x049
XK_J: Final = 0x04A
XK_K: Final = 0x04B
XK_L: Final = 0x04C
XK_M: Final = 0x04D
XK_N: Final = 0x04E
XK_O: Final = 0x04F
XK_P: Final = 0x050
XK_Q: Final = 0x051
XK_R: Final = 0x052
XK_S: Final = 0x053
XK_T: Final = 0x054
XK_U: Final = 0x055
XK_V: Final = 0x056
XK_W: Final = 0x057
XK_X: Final = 0x058
XK_Y: Final = 0x059
XK_Z: Final = 0x05A
XK_bracketleft: Final = 0x05B
XK_backslash: Final = 0x05C
XK_bracketright: Final = 0x05D
XK_asciicircum: Final = 0x05E
XK_underscore: Final = 0x05F
XK_grave: Final = 0x060
XK_quoteleft: Final = 0x060
XK_a: Final = 0x061
XK_b: Final = 0x062
XK_c: Final = 0x063
XK_d: Final = 0x064
XK_e: Final = 0x065
XK_f: Final = 0x066
XK_g: Final = 0x067
XK_h: Final = 0x068
XK_i: Final = 0x069
XK_j: Final = 0x06A
XK_k: Final = 0x06B
XK_l: Final = 0x06C
XK_m: Final = 0x06D
XK_n: Final = 0x06E
XK_o: Final = 0x06F
XK_p: Final = 0x070
XK_q: Final = 0x071
XK_r: Final = 0x072
XK_s: Final = 0x073
XK_t: Final = 0x074
XK_u: Final = 0x075
XK_v: Final = 0x076
XK_w: Final = 0x077
XK_x: Final = 0x078
XK_y: Final = 0x079
XK_z: Final = 0x07A
XK_braceleft: Final = 0x07B
XK_bar: Final = 0x07C
XK_braceright: Final = 0x07D
XK_asciitilde: Final = 0x07E
XK_nobreakspace: Final = 0x0A0
XK_exclamdown: Final = 0x0A1
XK_cent: Final = 0x0A2
XK_sterling: Final = 0x0A3
XK_currency: Final = 0x0A4
XK_yen: Final = 0x0A5
XK_brokenbar: Final = 0x0A6
XK_section: Final = 0x0A7
XK_diaeresis: Final = 0x0A8
XK_copyright: Final = 0x0A9
XK_ordfeminine: Final = 0x0AA
XK_guillemotleft: Final = 0x0AB
XK_notsign: Final = 0x0AC
XK_hyphen: Final = 0x0AD
XK_registered: Final = 0x0AE
XK_macron: Final = 0x0AF
XK_degree: Final = 0x0B0
XK_plusminus: Final = 0x0B1
XK_twosuperior: Final = 0x0B2
XK_threesuperior: Final = 0x0B3
XK_acute: Final = 0x0B4
XK_mu: Final = 0x0B5
XK_paragraph: Final = 0x0B6
XK_periodcentered: Final = 0x0B7
XK_cedilla: Final = 0x0B8
XK_onesuperior: Final = 0x0B9
XK_masculine: Final = 0x0BA
XK_guillemotright: Final = 0x0BB
XK_onequarter: Final = 0x0BC
XK_onehalf: Final = 0x0BD
XK_threequarters: Final = 0x0BE
XK_questiondown: Final = 0x0BF
XK_Agrave: Final = 0x0C0
XK_Aacute: Final = 0x0C1
XK_Acircumflex: Final = 0x0C2
XK_Atilde: Final = 0x0C3
XK_Adiaeresis: Final = 0x0C4
XK_Aring: Final = 0x0C5
XK_AE: Final = 0x0C6
XK_Ccedilla: Final = 0x0C7
XK_Egrave: Final = 0x0C8
XK_Eacute: Final = 0x0C9
XK_Ecircumflex: Final = 0x0CA
XK_Ediaeresis: Final = 0x0CB
XK_Igrave: Final = 0x0CC
XK_Iacute: Final = 0x0CD
XK_Icircumflex: Final = 0x0CE
XK_Idiaeresis: Final = 0x0CF
XK_ETH: Final = 0x0D0
XK_Eth: Final = 0x0D0
XK_Ntilde: Final = 0x0D1
XK_Ograve: Final = 0x0D2
XK_Oacute: Final = 0x0D3
XK_Ocircumflex: Final = 0x0D4
XK_Otilde: Final = 0x0D5
XK_Odiaeresis: Final = 0x0D6
XK_multiply: Final = 0x0D7
XK_Ooblique: Final = 0x0D8
XK_Ugrave: Final = 0x0D9
XK_Uacute: Final = 0x0DA
XK_Ucircumflex: Final = 0x0DB
XK_Udiaeresis: Final = 0x0DC
XK_Yacute: Final = 0x0DD
XK_THORN: Final = 0x0DE
XK_Thorn: Final = 0x0DE
XK_ssharp: Final = 0x0DF
XK_agrave: Final = 0x0E0
XK_aacute: Final = 0x0E1
XK_acircumflex: Final = 0x0E2
XK_atilde: Final = 0x0E3
XK_adiaeresis: Final = 0x0E4
XK_aring: Final = 0x0E5
XK_ae: Final = 0x0E6
XK_ccedilla: Final = 0x0E7
XK_egrave: Final = 0x0E8
XK_eacute: Final = 0x0E9
XK_ecircumflex: Final = 0x0EA
XK_ediaeresis: Final = 0x0EB
XK_igrave: Final = 0x0EC
XK_iacute: Final = 0x0ED
XK_icircumflex: Final = 0x0EE
XK_idiaeresis: Final = 0x0EF
XK_eth: Final = 0x0F0
XK_ntilde: Final = 0x0F1
XK_ograve: Final = 0x0F2
XK_oacute: Final = 0x0F3
XK_ocircumflex: Final = 0x0F4
XK_otilde: Final = 0x0F5
XK_odiaeresis: Final = 0x0F6
XK_division: Final = 0x0F7
XK_oslash: Final = 0x0F8
XK_ugrave: Final = 0x0F9
XK_uacute: Final = 0x0FA
XK_ucircumflex: Final = 0x0FB
XK_udiaeresis: Final = 0x0FC
XK_yacute: Final = 0x0FD
XK_thorn: Final = 0x0FE
XK_ydiaeresis: Final = 0x0FF



from _typeshed import Incomplete
from typing import Any

CODE_INDENT: int
reHtmlBlockOpen: Any
reHtmlBlockClose: Any
reThematicBreak: Any
reMaybeSpecial: Any
reNonSpace: Any
reBulletListMarker: Any
reOrderedListMarker: Any
reATXHeadingMarker: Any
reCodeFence: Any
reClosingCodeFence: Any
reSetextHeadingLine: Any
reLineEnding: Any

def is_blank(s): ...
def is_space_or_tab(s): ...
def peek(ln, pos): ...
def ends_with_blank_line(block): ...
def parse_list_marker(parser, container): ...
def lists_match(list_data, item_data): ...

class Block:
    accepts_lines: Any
    @staticmethod
    def continue_(parser: Incomplete | None = ..., container: Incomplete | None = ...) -> None: ...
    @staticmethod
    def finalize(parser: Incomplete | None = ..., block: Incomplete | None = ...) -> None: ...
    @staticmethod
    def can_contain(t) -> None: ...

class Document(Block):
    accepts_lines: bool
    @staticmethod
    def continue_(parser: Incomplete | None = ..., container: Incomplete | None = ...): ...
    @staticmethod
    def finalize(parser: Incomplete | None = ..., block: Incomplete | None = ...) -> None: ...
    @staticmethod
    def can_contain(t): ...

class List(Block):
    accepts_lines: bool
    @staticmethod
    def continue_(parser: Incomplete | None = ..., container: Incomplete | None = ...): ...
    @staticmethod
    def finalize(parser: Incomplete | None = ..., block: Incomplete | None = ...) -> None: ...
    @staticmethod
    def can_contain(t): ...

class BlockQuote(Block):
    accepts_lines: bool
    @staticmethod
    def continue_(parser: Incomplete | None = ..., container: Incomplete | None = ...): ...
    @staticmethod
    def finalize(parser: Incomplete | None = ..., block: Incomplete | None = ...) -> None: ...
    @staticmethod
    def can_contain(t): ...

class Item(Block):
    accepts_lines: bool
    @staticmethod
    def continue_(parser: Incomplete | None = ..., container: Incomplete | None = ...): ...
    @staticmethod
    def finalize(parser: Incomplete | None = ..., block: Incomplete | None = ...) -> None: ...
    @staticmethod
    def can_contain(t): ...

class Heading(Block):
    accepts_lines: bool
    @staticmethod
    def continue_(parser: Incomplete | None = ..., container: Incomplete | None = ...): ...
    @staticmethod
    def finalize(parser: Incomplete | None = ..., block: Incomplete | None = ...) -> None: ...
    @staticmethod
    def can_contain(t): ...

class ThematicBreak(Block):
    accepts_lines: bool
    @staticmethod
    def continue_(parser: Incomplete | None = ..., container: Incomplete | None = ...): ...
    @staticmethod
    def finalize(parser: Incomplete | None = ..., block: Incomplete | None = ...) -> None: ...
    @staticmethod
    def can_contain(t): ...

class CodeBlock(Block):
    accepts_lines: bool
    @staticmethod
    def continue_(parser: Incomplete | None = ..., container: Incomplete | None = ...): ...
    @staticmethod
    def finalize(parser: Incomplete | None = ..., block: Incomplete | None = ...) -> None: ...
    @staticmethod
    def can_contain(t): ...

class HtmlBlock(Block):
    accepts_lines: bool
    @staticmethod
    def continue_(parser: Incomplete | None = ..., container: Incomplete | None = ...): ...
    @staticmethod
    def finalize(parser: Incomplete | None = ..., block: Incomplete | None = ...) -> None: ...
    @staticmethod
    def can_contain(t): ...

class Paragraph(Block):
    accepts_lines: bool
    @staticmethod
    def continue_(parser: Incomplete | None = ..., container: Incomplete | None = ...): ...
    @staticmethod
    def finalize(parser: Incomplete | None = ..., block: Incomplete | None = ...) -> None: ...
    @staticmethod
    def can_contain(t): ...

class BlockStarts:
    METHODS: Any
    @staticmethod
    def block_quote(parser, container: Incomplete | None = ...): ...
    @staticmethod
    def atx_heading(parser, container: Incomplete | None = ...): ...
    @staticmethod
    def fenced_code_block(parser, container: Incomplete | None = ...): ...
    @staticmethod
    def html_block(parser, container: Incomplete | None = ...): ...
    @staticmethod
    def setext_heading(parser, container: Incomplete | None = ...): ...
    @staticmethod
    def thematic_break(parser, container: Incomplete | None = ...): ...
    @staticmethod
    def list_item(parser, container: Incomplete | None = ...): ...
    @staticmethod
    def indented_code_block(parser, container: Incomplete | None = ...): ...

class Parser:
    doc: Any
    block_starts: Any
    tip: Any
    oldtip: Any
    current_line: str
    line_number: int
    offset: int
    column: int
    next_nonspace: int
    next_nonspace_column: int
    indent: int
    indented: bool
    blank: bool
    partially_consumed_tab: bool
    all_closed: bool
    last_matched_container: Any
    refmap: Any
    last_line_length: int
    inline_parser: Any
    options: Any
    def __init__(self, options=...) -> None: ...
    def add_line(self) -> None: ...
    def add_child(self, tag, offset): ...
    def close_unmatched_blocks(self) -> None: ...
    def find_next_nonspace(self) -> None: ...
    def advance_next_nonspace(self) -> None: ...
    def advance_offset(self, count, columns) -> None: ...
    def incorporate_line(self, ln) -> None: ...
    def finalize(self, block, line_number) -> None: ...
    def process_inlines(self, block) -> None: ...
    def parse(self, my_input): ...

CAMEL_RE: Any



import types
import zipimport
from _typeshed import Incomplete, StrPath, Unused
from collections.abc import Callable, Generator, Iterable, Iterator, Sequence
from io import BytesIO
from itertools import chain
from pkgutil import get_importer as get_importer
from re import Pattern
from typing import IO, Any, ClassVar, Final, Literal, NoReturn, Protocol, TypeVar, overload, type_check_only
from typing_extensions import Self, TypeAlias
from zipfile import ZipInfo

from ._vendored_packaging import requirements as packaging_requirements, version as packaging_version

_T = TypeVar("_T")
_D = TypeVar("_D", bound=Distribution)
_NestedStr: TypeAlias = str | Iterable[_NestedStr]
_StrictInstallerType: TypeAlias = Callable[[Requirement], _D]
_InstallerType: TypeAlias = Callable[[Requirement], Distribution | None] | None
_PkgReqType: TypeAlias = str | Requirement
_EPDistType: TypeAlias = Distribution | _PkgReqType
_MetadataType: TypeAlias = IResourceProvider | None
_ResolvedEntryPoint: TypeAlias = Any  # Can be any attribute in the module
_ModuleLike: TypeAlias = object | types.ModuleType  # Any object that optionally has __loader__ or __file__, usually a module
_ProviderFactoryType: TypeAlias = Callable[[_ModuleLike], IResourceProvider]
_DistFinderType: TypeAlias = Callable[[_T, str, bool], Iterable[Distribution]]
_NSHandlerType: TypeAlias = Callable[[_T, str, str, types.ModuleType], str | None]
_ResourceStream: TypeAlias = Incomplete  # A readable file-like object

__all__ = [
    "require",
    "run_script",
    "get_provider",
    "get_distribution",
    "load_entry_point",
    "get_entry_map",
    "get_entry_info",
    "iter_entry_points",
    "resource_string",
    "resource_stream",
    "resource_filename",
    "resource_listdir",
    "resource_exists",
    "resource_isdir",
    "declare_namespace",
    "working_set",
    "add_activation_listener",
    "find_distributions",
    "set_extraction_path",
    "cleanup_resources",
    "get_default_cache",
    "Environment",
    "WorkingSet",
    "ResourceManager",
    "Distribution",
    "Requirement",
    "EntryPoint",
    "ResolutionError",
    "VersionConflict",
    "DistributionNotFound",
    "UnknownExtra",
    "ExtractionError",
    "PEP440Warning",
    "parse_requirements",
    "parse_version",
    "safe_name",
    "safe_version",
    "get_platform",
    "compatible_platforms",
    "yield_lines",
    "split_sections",
    "safe_extra",
    "to_filename",
    "invalid_marker",
    "evaluate_marker",
    "ensure_directory",
    "normalize_path",
    "EGG_DIST",
    "BINARY_DIST",
    "SOURCE_DIST",
    "CHECKOUT_DIST",
    "DEVELOP_DIST",
    "IMetadataProvider",
    "IResourceProvider",
    "FileMetadata",
    "PathMetadata",
    "EggMetadata",
    "EmptyProvider",
    "empty_provider",
    "NullProvider",
    "EggProvider",
    "DefaultProvider",
    "ZipProvider",
    "register_finder",
    "register_namespace_handler",
    "register_loader_type",
    "fixup_namespace_packages",
    "get_importer",
    "PkgResourcesDeprecationWarning",
    "run_main",
    "AvailableDistributions",
]

@type_check_only
class _ZipLoaderModule(Protocol):
    __loader__: zipimport.zipimporter

def declare_namespace(packageName: str) -> None: ...
def fixup_namespace_packages(path_item: str, parent: str | None = None) -> None: ...

class WorkingSet:
    entries: list[str]
    def __init__(self, entries: Iterable[str] | None = None) -> None: ...
    def add_entry(self, entry: str) -> None: ...
    def __contains__(self, dist: Distribution) -> bool: ...
    def find(self, req: Requirement) -> Distribution | None: ...
    def iter_entry_points(self, group: str, name: str | None = None) -> Generator[EntryPoint, None, None]: ...
    def run_script(self, requires: str, script_name: str) -> None: ...
    def __iter__(self) -> Iterator[Distribution]: ...
    def add(self, dist: Distribution, entry: str | None = None, insert: bool = True, replace: bool = False) -> None: ...
    @overload
    def resolve(  # type: ignore[overload-overlap]
        self,
        requirements: Iterable[Requirement],
        env: Environment | None,
        installer: _StrictInstallerType[_D],
        replace_conflicting: bool = False,
        extras: tuple[str, ...] | None = None,
    ) -> list[_D]: ...
    @overload
    def resolve(  # type: ignore[overload-overlap]
        self,
        requirements: Iterable[Requirement],
        env: Environment | None = None,
        *,
        installer: _StrictInstallerType[_D],
        replace_conflicting: bool = False,
        extras: tuple[str, ...] | None = None,
    ) -> list[_D]: ...
    @overload
    def resolve(
        self,
        requirements: Iterable[Requirement],
        env: Environment | None = None,
        installer: _InstallerType = None,
        replace_conflicting: bool = False,
        extras: tuple[str, ...] | None = None,
    ) -> list[Distribution]: ...
    @overload
    def find_plugins(  # type: ignore[overload-overlap]
        self, plugin_env: Environment, full_env: Environment | None, installer: _StrictInstallerType[_D], fallback: bool = True
    ) -> tuple[list[_D], dict[Distribution, Exception]]: ...
    @overload
    def find_plugins(  # type: ignore[overload-overlap]
        self,
        plugin_env: Environment,
        full_env: Environment | None = None,
        *,
        installer: _StrictInstallerType[_D],
        fallback: bool = True,
    ) -> tuple[list[_D], dict[Distribution, Exception]]: ...
    @overload
    def find_plugins(
        self,
        plugin_env: Environment,
        full_env: Environment | None = None,
        installer: _InstallerType = None,
        fallback: bool = True,
    ) -> tuple[list[Distribution], dict[Distribution, Exception]]: ...
    def require(self, *requirements: _NestedStr) -> Sequence[Distribution]: ...
    def subscribe(self, callback: Callable[[Distribution], object], existing: bool = True) -> None: ...

working_set: WorkingSet
require = working_set.require
iter_entry_points = working_set.iter_entry_points
add_activation_listener = working_set.subscribe
run_script = working_set.run_script
run_main = run_script

class Environment:
    def __init__(
        self, search_path: Sequence[str] | None = None, platform: str | None = ..., python: str | None = ...
    ) -> None: ...
    def can_add(self, dist: Distribution) -> bool: ...
    def remove(self, dist: Distribution) -> None: ...
    def scan(self, search_path: Sequence[str] | None = None) -> None: ...
    def __getitem__(self, project_name: str) -> list[Distribution]: ...
    def add(self, dist: Distribution) -> None: ...
    @overload
    def best_match(
        self, req: Requirement, working_set: WorkingSet, installer: _StrictInstallerType[_D], replace_conflicting: bool = False
    ) -> _D: ...
    @overload
    def best_match(
        self, req: Requirement, working_set: WorkingSet, installer: _InstallerType = None, replace_conflicting: bool = False
    ) -> Distribution | None: ...
    @overload
    def obtain(self, requirement: Requirement, installer: _StrictInstallerType[_D]) -> _D: ...  # type: ignore[overload-overlap]
    @overload
    def obtain(self, requirement: Requirement, installer: Callable[[Requirement], None] | None = None) -> None: ...
    @overload
    def obtain(self, requirement: Requirement, installer: _InstallerType = None) -> Distribution | None: ...
    def __iter__(self) -> Iterator[str]: ...
    def __iadd__(self, other: Distribution | Environment) -> Self: ...
    def __add__(self, other: Distribution | Environment) -> Self: ...

AvailableDistributions = Environment

def parse_requirements(strs: _NestedStr) -> Iterator[Requirement]: ...

class RequirementParseError(packaging_requirements.InvalidRequirement): ...

class Requirement(packaging_requirements.Requirement):
    unsafe_name: str
    project_name: str
    key: str
    # packaging.requirements.Requirement uses a set for its extras. setuptools/pkg_resources uses a variable-length tuple
    extras: tuple[str, ...]  # type: ignore[assignment]
    specs: list[tuple[str, str]]
    def __init__(self, requirement_string: str) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __contains__(self, item: Distribution | str | tuple[str, ...]) -> bool: ...
    @staticmethod
    def parse(s: str | Iterable[str]) -> Requirement: ...

def load_entry_point(dist: _EPDistType, group: str, name: str) -> _ResolvedEntryPoint: ...
@overload
def get_entry_map(dist: _EPDistType, group: None = None) -> dict[str, dict[str, EntryPoint]]: ...
@overload
def get_entry_map(dist: _EPDistType, group: str) -> dict[str, EntryPoint]: ...
def get_entry_info(dist: _EPDistType, group: str, name: str) -> EntryPoint | None: ...

class EntryPoint:
    name: str
    module_name: str
    attrs: tuple[str, ...]
    extras: tuple[str, ...]
    dist: Distribution | None
    def __init__(
        self, name: str, module_name: str, attrs: Iterable[str] = (), extras: Iterable[str] = (), dist: Distribution | None = None
    ) -> None: ...
    @overload
    def load(
        self, require: Literal[True] = True, env: Environment | None = None, installer: _InstallerType = None
    ) -> _ResolvedEntryPoint: ...
    @overload
    def load(self, require: Literal[False], *args: Unused, **kwargs: Unused) -> _ResolvedEntryPoint: ...
    def resolve(self) -> _ResolvedEntryPoint: ...
    def require(self, env: Environment | None = None, installer: _InstallerType = None) -> None: ...
    pattern: ClassVar[Pattern[str]]
    @classmethod
    def parse(cls, src: str, dist: Distribution | None = None) -> Self: ...
    @classmethod
    def parse_group(cls, group: str, lines: _NestedStr, dist: Distribution | None = None) -> dict[str, Self]: ...
    @classmethod
    def parse_map(
        cls, data: str | Iterable[str] | dict[str, str | Iterable[str]], dist: Distribution | None = None
    ) -> dict[str, dict[str, Self]]: ...

def find_distributions(path_item: str, only: bool = False) -> Generator[Distribution, None, None]: ...
@overload
def get_distribution(dist: _D) -> _D: ...
@overload
def get_distribution(dist: _PkgReqType) -> Distribution: ...

PY_MAJOR: Final[str]
EGG_DIST: Final = 3
BINARY_DIST: Final = 2
SOURCE_DIST: Final = 1
CHECKOUT_DIST: Final = 0
DEVELOP_DIST: Final = -1

class ResourceManager:
    extraction_path: str | None
    cached_files: Incomplete
    def resource_exists(self, package_or_requirement: _PkgReqType, resource_name: str) -> bool: ...
    def resource_isdir(self, package_or_requirement: _PkgReqType, resource_name: str) -> bool: ...
    def resource_filename(self, package_or_requirement: _PkgReqType, resource_name: str) -> str: ...
    def resource_stream(self, package_or_requirement: _PkgReqType, resource_name: str) -> IO[bytes]: ...
    def resource_string(self, package_or_requirement: _PkgReqType, resource_name: str) -> bytes: ...
    def resource_listdir(self, package_or_requirement: _PkgReqType, resource_name: str) -> list[str]: ...
    def extraction_error(self) -> NoReturn: ...
    def get_cache_path(self, archive_name: str, names: Iterable[str] = ()) -> str: ...
    def postprocess(self, tempname: str, filename: str) -> None: ...
    def set_extraction_path(self, path: str) -> None: ...
    def cleanup_resources(self, force: bool = False) -> list[str]: ...

__resource_manager: ResourceManager  # Doesn't exist at runtime
resource_exists = __resource_manager.resource_exists
resource_isdir = __resource_manager.resource_isdir
resource_filename = __resource_manager.resource_filename
resource_stream = __resource_manager.resource_stream
resource_string = __resource_manager.resource_string
resource_listdir = __resource_manager.resource_listdir
set_extraction_path = __resource_manager.set_extraction_path
cleanup_resources = __resource_manager.cleanup_resources

@overload
def get_provider(moduleOrReq: str) -> IResourceProvider: ...
@overload
def get_provider(moduleOrReq: Requirement) -> Distribution: ...

class IMetadataProvider(Protocol):
    def has_metadata(self, name: str) -> bool: ...
    def get_metadata(self, name: str) -> str: ...
    def get_metadata_lines(self, name: str) -> Iterator[str]: ...
    def metadata_isdir(self, name: str) -> bool: ...
    def metadata_listdir(self, name: str) -> list[str]: ...
    def run_script(self, script_name: str, namespace: dict[str, Any]) -> None: ...

class ResolutionError(Exception): ...

class DistributionNotFound(ResolutionError):
    def __init__(self, req: Requirement, requirers: set[str] | None, /, *args: object) -> None: ...
    @property
    def req(self) -> Requirement: ...
    @property
    def requirers(self) -> set[str] | None: ...
    @property
    def requirers_str(self) -> str: ...
    def report(self) -> str: ...

class VersionConflict(ResolutionError):
    def __init__(self, dist: Distribution, req: Requirement, /, *args: object) -> None: ...
    @property
    def dist(self) -> Distribution: ...
    @property
    def req(self) -> Requirement: ...
    def report(self) -> str: ...
    def with_context(self, required_by: set[str]) -> Self | ContextualVersionConflict: ...

class ContextualVersionConflict(VersionConflict):
    def __init__(self, dist: Distribution, req: Requirement, required_by: set[str], /, *args: object) -> None: ...
    @property
    def required_by(self) -> set[str]: ...

class UnknownExtra(ResolutionError): ...

class ExtractionError(Exception):
    manager: ResourceManager
    cache_path: str
    original_error: BaseException | None

def register_finder(importer_type: type[_T], distribution_finder: _DistFinderType[_T]) -> None: ...
def register_loader_type(loader_type: type[_ModuleLike], provider_factory: _ProviderFactoryType) -> None: ...
def register_namespace_handler(importer_type: type[_T], namespace_handler: _NSHandlerType[_T]) -> None: ...

class IResourceProvider(IMetadataProvider, Protocol):
    def get_resource_filename(self, manager: ResourceManager, resource_name: str) -> StrPath: ...
    def get_resource_stream(self, manager: ResourceManager, resource_name: str) -> _ResourceStream: ...
    def get_resource_string(self, manager: ResourceManager, resource_name: str) -> bytes: ...
    def has_resource(self, resource_name: str) -> bool: ...
    def resource_isdir(self, resource_name: str) -> bool: ...
    def resource_listdir(self, resource_name: str) -> list[str]: ...

def invalid_marker(text: str) -> SyntaxError | Literal[False]: ...
def evaluate_marker(text: str, extra: Incomplete | None = None) -> bool: ...

class NullProvider:
    egg_name: str | None
    egg_info: str | None
    loader: types._LoaderProtocol | None
    module_path: str | None

    def __init__(self, module: _ModuleLike) -> None: ...
    def get_resource_filename(self, manager: ResourceManager, resource_name: str) -> str: ...
    def get_resource_stream(self, manager: ResourceManager, resource_name: str) -> BytesIO: ...
    def get_resource_string(self, manager: ResourceManager, resource_name: str) -> bytes: ...
    def has_resource(self, resource_name: str) -> bool: ...
    def has_metadata(self, name: str) -> bool: ...
    def get_metadata(self, name: str) -> str: ...
    def get_metadata_lines(self, name: str) -> chain[str]: ...
    def resource_isdir(self, resource_name: str) -> bool: ...
    def metadata_isdir(self, name: str) -> bool: ...
    def resource_listdir(self, resource_name: str) -> list[str]: ...
    def metadata_listdir(self, name: str) -> list[str]: ...
    def run_script(self, script_name: str, namespace: dict[str, Any]) -> None: ...

# Doesn't actually extend NullProvider, solves a typing issue in pytype_test.py
class Distribution(NullProvider):
    PKG_INFO: ClassVar[str]
    project_name: str
    py_version: str | None
    platform: str | None
    location: str | None
    precedence: int
    def __init__(
        self,
        location: str | None = None,
        metadata: _MetadataType = None,
        project_name: str | None = None,
        version: str | None = None,
        py_version: str | None = ...,
        platform: str | None = None,
        precedence: int = 3,
    ) -> None: ...
    @classmethod
    def from_location(
        cls, location: str, basename: str, metadata: _MetadataType = None, *, precedence: int = 3
    ) -> Distribution: ...
    @property
    def hashcmp(self) -> tuple[parse_version, int, str, str | None, str, str]: ...
    def __hash__(self) -> int: ...
    def __lt__(self, other: Distribution) -> bool: ...
    def __le__(self, other: Distribution) -> bool: ...
    def __gt__(self, other: Distribution) -> bool: ...
    def __ge__(self, other: Distribution) -> bool: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    @property
    def key(self) -> str: ...
    @property
    def parsed_version(self) -> packaging_version.Version: ...
    @property
    def version(self) -> str: ...
    def requires(self, extras: Iterable[str] = ()) -> list[Requirement]: ...
    def activate(self, path: list[str] | None = None, replace: bool = False) -> None: ...
    def egg_name(self) -> str: ...  # type: ignore[override]  # supertype's egg_name is a variable, not a method
    @classmethod
    def from_filename(cls, filename: str, metadata: _MetadataType = None, *, precedence: int = 3) -> Distribution: ...
    def as_requirement(self) -> Requirement: ...
    def load_entry_point(self, group: str, name: str) -> _ResolvedEntryPoint: ...
    @overload
    def get_entry_map(self, group: None = None) -> dict[str, dict[str, EntryPoint]]: ...
    @overload
    def get_entry_map(self, group: str) -> dict[str, EntryPoint]: ...
    def get_entry_info(self, group: str, name: str) -> EntryPoint | None: ...
    def insert_on(self, path: list[str], loc: Incomplete | None = None, replace: bool = False) -> None: ...
    def check_version_conflict(self) -> None: ...
    def has_version(self) -> bool: ...
    def clone(self, **kw: str | int | IResourceProvider | None) -> Requirement: ...
    @property
    def extras(self) -> list[str]: ...

class DistInfoDistribution(Distribution):
    PKG_INFO: ClassVar[Literal["METADATA"]]
    EQEQ: ClassVar[Pattern[str]]

class EggProvider(NullProvider):
    egg_root: str

class DefaultProvider(EggProvider): ...

class PathMetadata(DefaultProvider):
    egg_info: str
    module_path: str
    def __init__(self, path: str, egg_info: str) -> None: ...

class ZipProvider(EggProvider):
    eagers: list[str] | None
    zip_pre: str
    # ZipProvider's loader should always be a zipimporter
    loader: zipimport.zipimporter
    def __init__(self, module: _ZipLoaderModule) -> None: ...
    @property
    def zipinfo(self) -> dict[str, ZipInfo]: ...

class EggMetadata(ZipProvider):
    loader: zipimport.zipimporter
    module_path: str
    def __init__(self, importer: zipimport.zipimporter) -> None: ...

class EmptyProvider(NullProvider):
    module_path: None
    def __init__(self) -> None: ...

empty_provider: EmptyProvider

class FileMetadata(EmptyProvider):
    path: StrPath
    def __init__(self, path: StrPath) -> None: ...

class PEP440Warning(RuntimeWarning): ...

parse_version = packaging_version.Version

def yield_lines(iterable: _NestedStr) -> chain[str]: ...
def split_sections(s: _NestedStr) -> Generator[tuple[str | None, list[str]], None, None]: ...
def safe_name(name: str) -> str: ...
def safe_version(version: str) -> str: ...
def safe_extra(extra: str) -> str: ...
def to_filename(name: str) -> str: ...
def get_build_platform() -> str: ...

get_platform = get_build_platform

def get_supported_platform() -> str: ...
def compatible_platforms(provided: str | None, required: str | None) -> bool: ...
def get_default_cache() -> str: ...
def ensure_directory(path: str) -> None: ...
def normalize_path(filename: str) -> str: ...

class PkgResourcesDeprecationWarning(Warning): ...



from hvac.utils import _DeprecateProperty

DEPRECATED_PROPERTIES: dict[str, _DeprecateProperty]
DEFAULT_URL: str
VAULT_CACERT: str | None
VAULT_CAPATH: str | None
VAULT_CLIENT_CERT: str | None
VAULT_CLIENT_KEY: str | None



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def weisfeiler_lehman_graph_hash(
    G, edge_attr: Incomplete | None = None, node_attr: Incomplete | None = None, iterations: int = 3, digest_size: int = 16
): ...
@_dispatch
def weisfeiler_lehman_subgraph_hashes(
    G, edge_attr: Incomplete | None = None, node_attr: Incomplete | None = None, iterations: int = 3, digest_size: int = 16
): ...



from _typeshed import Incomplete
from typing import ClassVar

from ..core import WesternCalendar

class Barbados(WesternCalendar):
    include_labour_day: ClassVar[bool]
    include_good_friday: ClassVar[bool]
    include_easter_sunday: ClassVar[bool]
    include_easter_monday: ClassVar[bool]
    include_whit_monday: ClassVar[bool]
    non_computable_holiday_dict: Incomplete
    FIXED_HOLIDAYS: Incomplete
    def get_kadooment_day(self, year): ...
    def get_emancipation_day(self, year): ...
    def get_variable_days(self, year): ...
    def non_computable_holiday(self, year): ...
    def get_fixed_holidays(self, year): ...
